# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from ziggurat_foundations.models.base import get_db_session


class BaseService(object):
    model = None
    models_proxy = None

    @classmethod
    def all(cls, klass, db_session=None):
        """
        returns all objects of specific type - will work correctly with
        sqlalchemy inheritance models, you should normally use models
        base_query()  instead of this function its for bw. compat purposes

        :param klass:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return db_session.query(klass)

    @classmethod
    def base_query(cls, db_session=None):
        """
        returns base query for specific service

        :param db_session:
        :return: query
        """
        db_session = get_db_session(db_session)
        return db_session.query(cls.model)



"""add registered_date to user

Revision ID: 54d08f9adc8c
Revises: 2d472fe79b95
Create Date: 2012-03-10 11:12:39.353857

"""
from __future__ import unicode_literals

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects.postgresql.base import PGDialect

# revision identifiers, used by Alembic.
revision = "54d08f9adc8c"
down_revision = "2d472fe79b95"


def upgrade():
    from alembic.context import get_context

    c = get_context()
    if isinstance(c.connection.engine.dialect, PGDialect):
        op.add_column(
            "users",
            sa.Column(
                "registered_date",
                sa.TIMESTAMP(timezone=False),
                default=sa.sql.func.now(),
                server_default=sa.func.now(),
            ),
        )
    else:
        op.add_column(
            "users",
            sa.Column(
                "registered_date",
                sa.TIMESTAMP(timezone=False),
                default=sa.sql.func.now(),
            ),
        )


def downgrade():
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from ziggurat_foundations.models.base import get_db_session
from ziggurat_foundations.models.services import BaseService

__all__ = ["GroupPermissionService"]


class GroupPermissionService(BaseService):
    @classmethod
    def get(cls, group_id, perm_name, db_session=None):
        """
        Fetch row using primary key -
        will use existing object in session if already present

        :param group_id:
        :param perm_name:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return db_session.query(cls.model).get([group_id, perm_name])

    @classmethod
    def by_group_and_perm(cls, group_id, perm_name, db_session=None):
        """
        return by by_user_and_perm and permission name

        :param group_id:
        :param perm_name:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        query = db_session.query(cls.model).filter(cls.model.group_id == group_id)
        query = query.filter(cls.model.perm_name == perm_name)
        return query.first()



"""bigger identity datatypes

Revision ID: 2d472fe79b95
Revises: 264049f80948
Create Date: 2012-02-19 17:24:24.422312

"""
from __future__ import unicode_literals

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "2d472fe79b95"
down_revision = "264049f80948"


def upgrade():
    op.alter_column(
        "external_identities",
        "external_id",
        type_=sa.Unicode(255),
        existing_type=sa.Unicode(50),
        nullable=False,
    )
    op.alter_column(
        "external_identities",
        "external_user_name",
        type_=sa.Unicode(255),
        existing_type=sa.Unicode(50),
    )


def downgrade():
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from collections import OrderedDict

import sqlalchemy as sa

from ziggurat_foundations.utils import noop
from ziggurat_foundations.exc import (
    ZigguratResourceTreeMissingException,
    ZigguratResourceTreePathException,
    ZigguratResourceOutOfBoundaryException,
)
from ziggurat_foundations.models.base import get_db_session
from ziggurat_foundations.models.services.resource import ResourceService

__all__ = ["ResourceTreeServicePostgreSQL"]


class ResourceTreeServicePostgreSQL(object):
    model = None

    @classmethod
    def from_resource_deeper(
        cls, resource_id=None, limit_depth=1000000, db_session=None, *args, **kwargs
    ):
        """
        This returns you subtree of ordered objects relative
        to the start resource_id (currently only implemented in postgresql)

        :param resource_id:
        :param limit_depth:
        :param db_session:
        :return:
        """
        tablename = cls.model.__table__.name
        raw_q = """
            WITH RECURSIVE subtree AS (
                    SELECT res.*, 1 AS depth, LPAD(res.ordering::CHARACTER VARYING, 7, '0') AS sorting,
                    res.resource_id::CHARACTER VARYING AS path
                    FROM {tablename} AS res WHERE res.resource_id = :resource_id
                  UNION ALL
                    SELECT res_u.*, depth+1 AS depth,
                    (st.sorting::CHARACTER VARYING || '/' || LPAD(res_u.ordering::CHARACTER VARYING, 7, '0') ) AS sorting,
                    (st.path::CHARACTER VARYING || '/' || res_u.resource_id::CHARACTER VARYING ) AS path
                    FROM {tablename} res_u, subtree st
                    WHERE res_u.parent_id = st.resource_id
            )
            SELECT * FROM subtree WHERE depth<=:depth ORDER BY sorting;
        """.format(
            tablename=tablename
        )  # noqa
        db_session = get_db_session(db_session)
        text_obj = sa.text(raw_q)
        query = db_session.query(cls.model, "depth", "sorting", "path")
        query = query.from_statement(text_obj)
        query = query.params(resource_id=resource_id, depth=limit_depth)
        return query

    @classmethod
    def delete_branch(cls, resource_id=None, db_session=None, *args, **kwargs):
        """
        This deletes whole branch with children starting from resource_id

        :param resource_id:
        :param db_session:
        :return:
        """
        tablename = cls.model.__table__.name

        # lets lock rows to prevent bad tree states
        resource = ResourceService.lock_resource_for_update(
            resource_id=resource_id, db_session=db_session
        )
        parent_id = resource.parent_id
        ordering = resource.ordering
        raw_q = """
            WITH RECURSIVE subtree AS (
                    SELECT res.resource_id
                    FROM {tablename} AS res WHERE res.resource_id = :resource_id
                  UNION ALL
                    SELECT res_u.resource_id
                    FROM {tablename} res_u, subtree st
                    WHERE res_u.parent_id = st.resource_id
            )
            DELETE FROM resources where resource_id in (select * from subtree);
        """.format(
            tablename=tablename
        )  # noqa
        db_session = get_db_session(db_session)
        text_obj = sa.text(raw_q)
        db_session.execute(text_obj, params={"resource_id": resource_id})
        cls.shift_ordering_down(parent_id, ordering, db_session=db_session)
        return True

    @classmethod
    def from_parent_deeper(
        cls, parent_id=None, limit_depth=1000000, db_session=None, *args, **kwargs
    ):
        """
        This returns you subtree of ordered objects relative
        to the start parent_id (currently only implemented in postgresql)

        :param resource_id:
        :param limit_depth:
        :param db_session:
        :return:
        """

        if parent_id:
            limiting_clause = "res.parent_id = :parent_id"
        else:
            limiting_clause = "res.parent_id is null"
        tablename = cls.model.__table__.name
        raw_q = """
            WITH RECURSIVE subtree AS (
                    SELECT res.*, 1 AS depth, LPAD(res.ordering::CHARACTER VARYING, 7, '0') AS sorting,
                    res.resource_id::CHARACTER VARYING AS path
                    FROM {tablename} AS res WHERE {limiting_clause}
                  UNION ALL
                    SELECT res_u.*, depth+1 AS depth,
                    (st.sorting::CHARACTER VARYING || '/' || LPAD(res_u.ordering::CHARACTER VARYING, 7, '0') ) AS sorting,
                    (st.path::CHARACTER VARYING || '/' || res_u.resource_id::CHARACTER VARYING ) AS path
                    FROM {tablename} res_u, subtree st
                    WHERE res_u.parent_id = st.resource_id
            )
            SELECT * FROM subtree WHERE depth<=:depth ORDER BY sorting;
        """.format(
            tablename=tablename, limiting_clause=limiting_clause
        )  # noqa
        db_session = get_db_session(db_session)
        text_obj = sa.text(raw_q)
        query = db_session.query(cls.model, "depth", "sorting", "path")
        query = query.from_statement(text_obj)
        query = query.params(parent_id=parent_id, depth=limit_depth)
        return query

    @classmethod
    def build_subtree_strut(self, result, *args, **kwargs):
        """
        Returns a dictionary in form of
        {node:Resource, children:{node_id: Resource}}

        :param result:
        :return:
        """
        items = list(result)
        root_elem = {"node": None, "children": OrderedDict()}
        if len(items) == 0:
            return root_elem
        for _, node in enumerate(items):
            new_elem = {"node": node.Resource, "children": OrderedDict()}
            path = list(map(int, node.path.split("/")))
            parent_node = root_elem
            normalized_path = path[:-1]
            if normalized_path:
                for path_part in normalized_path:
                    parent_node = parent_node["children"][path_part]
            parent_node["children"][new_elem["node"].resource_id] = new_elem
        return root_elem

    @classmethod
    def path_upper(
        cls, object_id, limit_depth=1000000, db_session=None, *args, **kwargs
    ):
        """
        This returns you path to root node starting from object_id
            currently only for postgresql

        :param object_id:
        :param limit_depth:
        :param db_session:
        :return:
        """
        tablename = cls.model.__table__.name
        raw_q = """
            WITH RECURSIVE subtree AS (
                    SELECT res.*, 1 as depth FROM {tablename} res
                    WHERE res.resource_id = :resource_id
                  UNION ALL
                    SELECT res_u.*, depth+1 as depth
                    FROM {tablename} res_u, subtree st
                    WHERE res_u.resource_id = st.parent_id
            )
            SELECT * FROM subtree WHERE depth<=:depth;
        """.format(
            tablename=tablename
        )
        db_session = get_db_session(db_session)
        q = (
            db_session.query(cls.model)
            .from_statement(sa.text(raw_q))
            .params(resource_id=object_id, depth=limit_depth)
        )
        return q

    @classmethod
    def move_to_position(
        cls,
        resource_id,
        to_position,
        new_parent_id=noop,
        db_session=None,
        *args,
        **kwargs
    ):
        """
        Moves node to new location in the tree

        :param resource_id: resource to move
        :param to_position: new position
        :param new_parent_id: new parent id
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        # lets lock rows to prevent bad tree states
        resource = ResourceService.lock_resource_for_update(
            resource_id=resource_id, db_session=db_session
        )
        ResourceService.lock_resource_for_update(
            resource_id=resource.parent_id, db_session=db_session
        )
        same_branch = False

        # reset if parent is same as old
        if new_parent_id == resource.parent_id:
            new_parent_id = noop

        if new_parent_id is not noop:
            cls.check_node_parent(resource_id, new_parent_id, db_session=db_session)
        else:
            same_branch = True

        if new_parent_id is noop:
            # it is not guaranteed that parent exists
            parent_id = resource.parent_id if resource else None
        else:
            parent_id = new_parent_id

        cls.check_node_position(
            parent_id, to_position, on_same_branch=same_branch, db_session=db_session
        )
        # move on same branch
        if new_parent_id is noop:
            order_range = list(sorted((resource.ordering, to_position)))
            move_down = resource.ordering > to_position

            query = db_session.query(cls.model)
            query = query.filter(cls.model.parent_id == parent_id)
            query = query.filter(cls.model.ordering.between(*order_range))
            if move_down:
                query.update(
                    {cls.model.ordering: cls.model.ordering + 1},
                    synchronize_session=False,
                )
            else:
                query.update(
                    {cls.model.ordering: cls.model.ordering - 1},
                    synchronize_session=False,
                )
            db_session.flush()
            db_session.expire(resource)
            resource.ordering = to_position
        # move between branches
        else:
            cls.shift_ordering_down(
                resource.parent_id, resource.ordering, db_session=db_session
            )
            cls.shift_ordering_up(new_parent_id, to_position, db_session=db_session)
            db_session.expire(resource)
            resource.parent_id = new_parent_id
            resource.ordering = to_position
            db_session.flush()
        return True

    @classmethod
    def shift_ordering_down(cls, parent_id, position, db_session=None, *args, **kwargs):
        """
        Shifts ordering to "close gaps" after node deletion or being moved
        to another branch, begins the shift from given position

        :param parent_id:
        :param position:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        query = db_session.query(cls.model)
        query = query.filter(cls.model.parent_id == parent_id)
        query = query.filter(cls.model.ordering >= position)
        query.update(
            {cls.model.ordering: cls.model.ordering - 1}, synchronize_session=False
        )
        db_session.flush()

    @classmethod
    def shift_ordering_up(cls, parent_id, position, db_session=None, *args, **kwargs):
        """
        Shifts ordering to "open a gap" for node insertion,
        begins the shift from given position

        :param parent_id:
        :param position:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        query = db_session.query(cls.model)
        query = query.filter(cls.model.parent_id == parent_id)
        query = query.filter(cls.model.ordering >= position)
        query.update(
            {cls.model.ordering: cls.model.ordering + 1}, synchronize_session=False
        )
        db_session.flush()

    @classmethod
    def set_position(cls, resource_id, to_position, db_session=None, *args, **kwargs):
        """
        Sets node position for new node in the tree

        :param resource_id: resource to move
        :param to_position: new position
        :param db_session:
        :return:def count_children(cls, resource_id, db_session=None):
        """
        db_session = get_db_session(db_session)
        # lets lock rows to prevent bad tree states
        resource = ResourceService.lock_resource_for_update(
            resource_id=resource_id, db_session=db_session
        )
        cls.check_node_position(
            resource.parent_id, to_position, on_same_branch=True, db_session=db_session
        )
        cls.shift_ordering_up(resource.parent_id, to_position, db_session=db_session)
        db_session.flush()
        db_session.expire(resource)
        resource.ordering = to_position
        return True

    @classmethod
    def check_node_parent(
        cls, resource_id, new_parent_id, db_session=None, *args, **kwargs
    ):
        """
        Checks if parent destination is valid for node

        :param resource_id:
        :param new_parent_id:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        new_parent = ResourceService.lock_resource_for_update(
            resource_id=new_parent_id, db_session=db_session
        )
        # we are not moving to "root" so parent should be found
        if not new_parent and new_parent_id is not None:
            raise ZigguratResourceTreeMissingException("New parent node not found")
        else:
            result = cls.path_upper(new_parent_id, db_session=db_session)
            path_ids = [r.resource_id for r in result]
            if resource_id in path_ids:
                raise ZigguratResourceTreePathException(
                    "Trying to insert node into itself"
                )

    @classmethod
    def count_children(cls, resource_id, db_session=None, *args, **kwargs):
        """
        Counts children of resource node

        :param resource_id:
        :param db_session:
        :return:
        """
        query = db_session.query(cls.model.resource_id)
        query = query.filter(cls.model.parent_id == resource_id)
        return query.count()

    @classmethod
    def check_node_position(
        cls, parent_id, position, on_same_branch, db_session=None, *args, **kwargs
    ):
        """
        Checks if node position for given parent is valid, raises exception if
        this is not the case

        :param parent_id:
        :param position:
        :param on_same_branch: indicates that we are checking same branch
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        if not position or position < 1:
            raise ZigguratResourceOutOfBoundaryException(
                "Position is lower than {}", value=1
            )
        item_count = cls.count_children(parent_id, db_session=db_session)
        max_value = item_count if on_same_branch else item_count + 1
        if position > max_value:
            raise ZigguratResourceOutOfBoundaryException(
                "Maximum resource ordering is {}", value=max_value
            )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr

from ziggurat_foundations.exc import ZigguratException
from ziggurat_foundations.models.base import BaseModel

__all__ = ["ResourceMixin"]


class ResourceMixin(BaseModel):
    """
    Mixin for Resource model
    """

    __possible_permissions__ = ()

    @declared_attr
    def __tablename__(self):
        return "resources"

    @declared_attr
    def resource_id(self):
        return sa.Column(
            sa.Integer(), primary_key=True, nullable=False, autoincrement=True
        )

    @declared_attr
    def parent_id(self):
        return sa.Column(
            sa.Integer(),
            sa.ForeignKey(
                "resources.resource_id", onupdate="CASCADE", ondelete="SET NULL"
            ),
        )

    @declared_attr
    def ordering(self):
        return sa.Column(sa.Integer(), default=0, nullable=False)

    @declared_attr
    def resource_name(self):
        return sa.Column(sa.Unicode(100), nullable=False)

    @declared_attr
    def resource_type(self):
        return sa.Column(sa.Unicode(30), nullable=False)

    @declared_attr
    def owner_group_id(self):
        return sa.Column(
            sa.Integer,
            sa.ForeignKey("groups.id", onupdate="CASCADE", ondelete="SET NULL"),
            index=True,
        )

    @declared_attr
    def owner_user_id(self):
        return sa.Column(
            sa.Integer,
            sa.ForeignKey("users.id", onupdate="CASCADE", ondelete="SET NULL"),
            index=True,
        )

    @declared_attr
    def group_permissions(self):
        """ returns all group permissions for this resource"""
        return sa.orm.relationship(
            "GroupResourcePermission",
            cascade="all, delete-orphan",
            passive_deletes=True,
            passive_updates=True,
        )

    @declared_attr
    def user_permissions(self):
        """ returns all user permissions for this resource"""
        return sa.orm.relationship(
            "UserResourcePermission",
            cascade="all, delete-orphan",
            passive_deletes=True,
            passive_updates=True,
        )

    @declared_attr
    def groups(self):
        """ returns all groups that have permissions for this resource"""
        return sa.orm.relationship(
            "Group",
            secondary="groups_resources_permissions",
            passive_deletes=True,
            passive_updates=True,
        )

    @declared_attr
    def users(self):
        """ returns all users that have permissions for this resource"""
        return sa.orm.relationship(
            "User",
            secondary="users_resources_permissions",
            passive_deletes=True,
            passive_updates=True,
        )

    __mapper_args__ = {"polymorphic_on": resource_type}
    __table_args__ = {"mysql_engine": "InnoDB", "mysql_charset": "utf8"}

    def __repr__(self):
        return "<Resource: %s, %s, id: %s position: %s>" % (
            self.resource_type,
            self.resource_name,
            self.resource_id,
            self.ordering,
        )

    @property
    def __acl__(self):
        raise ZigguratException("Model should implement __acl__")

    @sa.orm.validates("user_permissions", "group_permissions")
    def validate_permission(self, key, permission):
        """ validate if resource can have specific permission """
        if permission.perm_name not in self.__possible_permissions__:
            raise AssertionError(
                "perm_name is not one of {}".format(self.__possible_permissions__)
            )
        return permission



"""normalize constraint and key names
correct keys for pre 0.5.6 naming convention

Revision ID: 438c27ec1c9
Revises: 439766f6104d
Create Date: 2015-06-13 21:16:32.358778

"""
from __future__ import unicode_literals

from alembic import op
from alembic.context import get_context
from sqlalchemy.dialects.postgresql.base import PGDialect
from sqlalchemy.engine.reflection import Inspector

# revision identifiers, used by Alembic.
revision = "438c27ec1c9"
down_revision = "439766f6104d"


# correct keys for pre 0.5.6 naming convention


def upgrade():
    c = get_context()
    insp = Inspector.from_engine(c.connection.engine)
    # existing migration
    # pre naming convention keys
    groups_permissions_pkey = "groups_permissions_pkey"
    groups_pkey = "groups_pkey"
    groups_resources_permissions_pkey = "groups_resources_permissions_pkey"
    users_groups_pkey = "users_groups_pkey"
    users_permissions_pkey = "users_permissions_pkey"
    users_resources_permissions_pkey = "users_resources_permissions_pkey"

    if isinstance(c.connection.engine.dialect, PGDialect):
        op.execute(
            "ALTER INDEX groups_unique_group_name_key RENAME to ix_groups_uq_group_name_key"
        )  # noqa

        op.drop_constraint("groups_permissions_perm_name_check", "groups_permissions")
        op.execute(
            """
        ALTER TABLE groups_permissions
            ADD CONSTRAINT ck_groups_permissions_perm_name CHECK (perm_name::text = lower(perm_name::text));
        """
        )  # noqa

        op.drop_constraint(
            "groups_resources_permissions_perm_name_check",
            "groups_resources_permissions",
        )
        op.execute(
            """
        ALTER TABLE groups_resources_permissions
              ADD CONSTRAINT ck_groups_resources_permissions_perm_name CHECK (perm_name::text = lower(perm_name::text));
        """
        )  # noqa

        op.drop_constraint("user_permissions_perm_name_check", "users_permissions")
        op.execute(
            """
        ALTER TABLE users_permissions
          ADD CONSTRAINT ck_user_permissions_perm_name CHECK (perm_name::text = lower(perm_name::text));
        """
        )  # noqa

        op.drop_constraint(
            "users_resources_permissions_perm_name_check", "users_resources_permissions"
        )
        op.execute(
            """
        ALTER TABLE users_resources_permissions
          ADD CONSTRAINT ck_users_resources_permissions_perm_name CHECK (perm_name::text = lower(perm_name::text));
        """
        )  # noqa

        op.execute("ALTER INDEX users_email_key2 RENAME to ix_users_uq_lower_email")

        op.execute(
            "ALTER INDEX users_username_uq2 RENAME to ix_users_ux_lower_username"
        )  # noqa

        if (
            groups_permissions_pkey
            == insp.get_pk_constraint("groups_permissions")["name"]
        ):
            op.execute(
                "ALTER INDEX groups_permissions_pkey RENAME to pk_groups_permissions"
            )  # noqa

        if groups_pkey == insp.get_pk_constraint("groups")["name"]:
            op.execute("ALTER INDEX groups_pkey RENAME to pk_groups")

        if (
            groups_resources_permissions_pkey
            == insp.get_pk_constraint("groups_resources_permissions")["name"]
        ):
            op.execute(
                "ALTER INDEX groups_resources_permissions_pkey RENAME to pk_groups_resources_permissions"
            )  # noqa

        if users_groups_pkey == insp.get_pk_constraint("users_groups")["name"]:
            op.execute("ALTER INDEX users_groups_pkey RENAME to pk_users_groups")

        if (
            users_permissions_pkey
            == insp.get_pk_constraint("users_permissions")["name"]
        ):
            op.execute(
                "ALTER INDEX users_permissions_pkey RENAME to pk_users_permissions"
            )  # noqa

        if (
            users_resources_permissions_pkey
            == insp.get_pk_constraint("users_resources_permissions")["name"]
        ):
            op.execute(
                "ALTER INDEX users_resources_permissions_pkey RENAME to pk_users_resources_permissions"
            )  # noqa

        if (
            "external_identities_pkey"
            == insp.get_pk_constraint("external_identities")["name"]
        ):
            op.execute(
                "ALTER INDEX external_identities_pkey RENAME to pk_external_identities"
            )  # noqa

        if "external_identities_local_user_name_fkey" in [
            c["name"] for c in insp.get_foreign_keys("external_identities")  # noqa
        ]:
            op.drop_constraint(
                "external_identities_local_user_name_fkey",
                "external_identities",
                type_="foreignkey",
            )
            op.create_foreign_key(
                None,
                "external_identities",
                "users",
                remote_cols=["user_name"],
                local_cols=["local_user_name"],
                onupdate="CASCADE",
                ondelete="CASCADE",
            )

        if "groups_permissions_group_id_fkey" in [
            c["name"] for c in insp.get_foreign_keys("groups_permissions")
        ]:
            op.drop_constraint(
                "groups_permissions_group_id_fkey",
                "groups_permissions",
                type_="foreignkey",
            )
            op.create_foreign_key(
                None,
                "groups_permissions",
                "groups",
                remote_cols=["id"],
                local_cols=["group_id"],
                onupdate="CASCADE",
                ondelete="CASCADE",
            )

        if "groups_group_name_key" in [
            c["name"] for c in insp.get_unique_constraints("groups")
        ]:
            op.execute(
                "ALTER INDEX groups_group_name_key RENAME to uq_groups_group_name"
            )  # noqa

        if "groups_resources_permissions_group_id_fkey" in [
            c["name"]
            for c in insp.get_foreign_keys("groups_resources_permissions")  # noqa
        ]:
            op.drop_constraint(
                "groups_resources_permissions_group_id_fkey",
                "groups_resources_permissions",
                type_="foreignkey",
            )
            op.create_foreign_key(
                None,
                "groups_resources_permissions",
                "groups",
                remote_cols=["id"],
                local_cols=["group_id"],
                onupdate="CASCADE",
                ondelete="CASCADE",
            )

        if "groups_resources_permissions_resource_id_fkey" in [
            c["name"]
            for c in insp.get_foreign_keys("groups_resources_permissions")  # noqa
        ]:
            op.drop_constraint(
                "groups_resources_permissions_resource_id_fkey",
                "groups_resources_permissions",
                type_="foreignkey",
            )
            op.create_foreign_key(
                None,
                "groups_resources_permissions",
                "resources",
                remote_cols=["resource_id"],
                local_cols=["resource_id"],
                onupdate="CASCADE",
                ondelete="CASCADE",
            )

        if "resources_pkey" == insp.get_pk_constraint("resources")["name"]:
            op.execute("ALTER INDEX resources_pkey RENAME to pk_resources")

        if "resources_owner_group_id_fkey" in [
            c["name"] for c in insp.get_foreign_keys("resources")
        ]:
            op.drop_constraint(
                "resources_owner_group_id_fkey", "resources", type_="foreignkey"
            )
            op.create_foreign_key(
                None,
                "resources",
                "groups",
                remote_cols=["id"],
                local_cols=["owner_group_id"],
                onupdate="CASCADE",
                ondelete="SET NULL",
            )

        if "resources_owner_user_id_fkey" in [
            c["name"] for c in insp.get_foreign_keys("resources")
        ]:
            op.drop_constraint(
                "resources_owner_user_id_fkey", "resources", type_="foreignkey"
            )
            op.create_foreign_key(
                None,
                "resources",
                "users",
                remote_cols=["id"],
                local_cols=["owner_user_id"],
                onupdate="CASCADE",
                ondelete="SET NULL",
            )

        if "resources_parent_id_fkey" in [
            c["name"] for c in insp.get_foreign_keys("resources")
        ]:
            op.drop_constraint(
                "resources_parent_id_fkey", "resources", type_="foreignkey"
            )
            op.create_foreign_key(
                None,
                "resources",
                "resources",
                remote_cols=["resource_id"],
                local_cols=["parent_id"],
                onupdate="CASCADE",
                ondelete="SET NULL",
            )

        if "users_pkey" == insp.get_pk_constraint("users")["name"]:
            op.execute("ALTER INDEX users_pkey RENAME to pk_users")

        if "users_email_key" in [
            c["name"] for c in insp.get_unique_constraints("users")
        ]:
            op.execute("ALTER INDEX users_email_key RENAME to uq_users_email")

        if "users_user_name_key" in [
            c["name"] for c in insp.get_unique_constraints("users")
        ]:
            op.execute("ALTER INDEX users_user_name_key RENAME to uq_users_user_name")

        if "users_groups_group_id_fkey" in [
            c["name"] for c in insp.get_foreign_keys("users_groups")
        ]:
            op.drop_constraint(
                "users_groups_group_id_fkey", "users_groups", type_="foreignkey"
            )
            op.create_foreign_key(
                None,
                "users_groups",
                "groups",
                remote_cols=["id"],
                local_cols=["group_id"],
                onupdate="CASCADE",
                ondelete="CASCADE",
            )

        if "users_groups_user_id_fkey" in [
            c["name"] for c in insp.get_foreign_keys("users_groups")
        ]:
            op.drop_constraint(
                "users_groups_user_id_fkey", "users_groups", type_="foreignkey"
            )
            op.create_foreign_key(
                None,
                "users_groups",
                "users",
                remote_cols=["id"],
                local_cols=["user_id"],
                onupdate="CASCADE",
                ondelete="CASCADE",
            )

        if "users_permissions_user_id_fkey" in [
            c["name"] for c in insp.get_foreign_keys("users_permissions")
        ]:
            op.drop_constraint(
                "users_permissions_user_id_fkey",
                "users_permissions",
                type_="foreignkey",
            )
            op.create_foreign_key(
                None,
                "users_permissions",
                "users",
                remote_cols=["id"],
                local_cols=["user_id"],
                onupdate="CASCADE",
                ondelete="CASCADE",
            )

        if "users_resources_permissions_resource_id_fkey" in [
            c["name"]
            for c in insp.get_foreign_keys(  # noqa  # noqa
                "users_resources_permissions"
            )
        ]:
            op.drop_constraint(
                "users_resources_permissions_resource_id_fkey",
                "users_resources_permissions",
                type_="foreignkey",
            )
            op.create_foreign_key(
                None,
                "users_resources_permissions",
                "resources",
                remote_cols=["resource_id"],
                local_cols=["resource_id"],
                onupdate="CASCADE",
                ondelete="CASCADE",
            )

        if "users_resources_permissions_user_id_fkey" in [
            c["name"]
            for c in insp.get_foreign_keys("users_resources_permissions")  # noqa
        ]:
            op.drop_constraint(
                "users_resources_permissions_user_id_fkey",
                "users_resources_permissions",
                type_="foreignkey",
            )
            op.create_foreign_key(
                None,
                "users_resources_permissions",
                "users",
                remote_cols=["id"],
                local_cols=["user_id"],
                onupdate="CASCADE",
                ondelete="CASCADE",
            )


def downgrade():
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from datetime import datetime

import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr

from ziggurat_foundations.models.base import BaseModel

__all__ = ["UserMixin"]


class UserMixin(BaseModel):
    """ Base mixin for User object representation.
        It supplies all the basic functionality from password hash generation
        and matching to utility methods used for querying database for users
        and their permissions or resources they have access to. It is meant
        to be extended with other application specific properties"""

    __mapper_args__ = {}
    __table_args__ = {"mysql_engine": "InnoDB", "mysql_charset": "utf8"}

    @declared_attr
    def __tablename__(self):
        return "users"

    @declared_attr
    def id(self):
        """ Unique identifier of user object"""
        return sa.Column(sa.Integer, primary_key=True, autoincrement=True)

    @declared_attr
    def user_name(self):
        """ Unique user name user object"""
        return sa.Column(sa.Unicode(128), unique=True)

    @declared_attr
    def user_password(self):
        """ Password hash for user object """
        return sa.Column(sa.Unicode(256))

    @declared_attr
    def email(self):
        """ Email for user object """
        return sa.Column(sa.Unicode(100), nullable=False, unique=True)

    @declared_attr
    def status(self):
        """ Status of user object """
        return sa.Column(sa.SmallInteger(), nullable=False, default=1)

    @declared_attr
    def security_code(self):
        """ Security code user object (can be used for password reset etc. """
        return sa.Column(sa.Unicode(256), default="default")

    @declared_attr
    def last_login_date(self):
        """ Date of user's last login """
        return sa.Column(
            sa.TIMESTAMP(timezone=False),
            default=lambda x: datetime.utcnow(),
            server_default=sa.func.now(),
        )

    @declared_attr
    def registered_date(self):
        """ Date of user's registration """
        return sa.Column(
            sa.TIMESTAMP(timezone=False),
            default=lambda x: datetime.utcnow(),
            server_default=sa.func.now(),
        )

    @declared_attr
    def security_code_date(self):
        """ Date of user's security code update """
        return sa.Column(
            sa.TIMESTAMP(timezone=False),
            default=datetime(2000, 1, 1),
            server_default="2000-01-01 01:01",
        )

    def __repr__(self):
        return "<User: %s>" % self.user_name

    @declared_attr
    def groups_dynamic(self):
        """ returns dynamic relationship for groups - allowing for
        filtering of data """
        return sa.orm.relationship(
            "Group",
            secondary="users_groups",
            lazy="dynamic",
            passive_deletes=True,
            passive_updates=True,
        )

    @declared_attr
    def user_permissions(self):
        """
        returns all direct non-resource permissions for this user,
        allows to assign new permissions to user::

            user.user_permissions.append(resource)
        """
        return sa.orm.relationship(
            "UserPermission",
            cascade="all, delete-orphan",
            passive_deletes=True,
            passive_updates=True,
        )

    @declared_attr
    def resource_permissions(self):
        """ returns all direct resource permissions for this user """
        return sa.orm.relationship(
            "UserResourcePermission",
            cascade="all, delete-orphan",
            passive_deletes=True,
            passive_updates=True,
        )

    @declared_attr
    def resources(self):
        """ Returns all resources directly owned by user, can be used to assign
        ownership of new resources::

            user.resources.append(resource) """
        return sa.orm.relationship(
            "Resource",
            cascade="all",
            passive_deletes=True,
            passive_updates=True,
            backref="owner",
            lazy="dynamic",
        )

    @declared_attr
    def external_identities(self):
        """ dynamic relation for external identities for this user -
        allowing for filtering of data """
        return sa.orm.relationship(
            "ExternalIdentity",
            lazy="dynamic",
            cascade="all, delete-orphan",
            passive_deletes=True,
            passive_updates=True,
            backref="owner",
        )



from __future__ import unicode_literals

# should hold global scoped session
DBSession = None


def groupfinder(userid, request):
    """
    Default groupfinder implementaion for pyramid applications

    :param userid:
    :param request:
    :return:
    """
    if userid and hasattr(request, "user") and request.user:
        groups = ["group:%s" % g.id for g in request.user.groups]
        return groups
    return []



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr

from ziggurat_foundations.models.base import BaseModel


__all__ = ["ExternalIdentityMixin"]


class ExternalIdentityMixin(BaseModel):
    """
    Mixin for External Identity model - it represents oAuth(or other) accounts
    attached to your user object
    """

    __table_args__ = (
        sa.PrimaryKeyConstraint(
            "external_id",
            "local_user_id",
            "provider_name",
            name="pk_external_identities",
        ),
        {"mysql_engine": "InnoDB", "mysql_charset": "utf8"},
    )

    @declared_attr
    def __tablename__(self):
        return "external_identities"

    @declared_attr
    def external_id(self):
        return sa.Column(sa.Unicode(255), default="", primary_key=True)

    @declared_attr
    def external_user_name(self):
        return sa.Column(sa.Unicode(255), default="")

    @declared_attr
    def local_user_id(self):
        return sa.Column(
            sa.Integer,
            sa.ForeignKey("users.id", onupdate="CASCADE", ondelete="CASCADE"),
            primary_key=True,
        )

    @declared_attr
    def provider_name(self):
        return sa.Column(sa.Unicode(50), default="", primary_key=True)

    @declared_attr
    def access_token(self):
        return sa.Column(sa.Unicode(512), default="")

    @declared_attr
    def alt_token(self):
        return sa.Column(sa.Unicode(512), default="")

    @declared_attr
    def token_secret(self):
        return sa.Column(sa.Unicode(512), default="")



# -*- coding: utf-8 -*-
from __future__ import with_statement, unicode_literals

import pytest

from ziggurat_foundations.tests import BaseTestCase, add_user


class TestExtPyramidLogin(BaseTestCase):
    def test_logon_bad(self, pyramid_app):
        result = pyramid_app.get("/sign_in").json
        assert result["view"] == "bad_auth"
        result = pyramid_app.post(
            "/sign_in", params={"login": "foo", "password": "bar"}
        ).json
        assert result["view"] == "bad_auth"

    def test_logon_good(self, db_session, pyramid_app):
        for item in (
            {"user_name": "username1", "email": "email", "password": "pass1"},
            {"user_name": "username2", "email": "email2", "password": "pass2"},
        ):
            user = add_user(db_session, **item)
            db_session.add(user)
            db_session.flush()
        result = pyramid_app.post(
            "/sign_in", params={"login": "username1", "password": "pass1"}
        ).json
        assert result["view"] == "sign_in"
        assert result["username"] == "username1"

    @pytest.mark.parametrize(
        "username,password", [("username1", "BAD"), ("username", "password")]
    )
    def test_logon_bad_pass(self, db_session, pyramid_app, username, password):
        user = add_user(db_session, user_name="username1", email="email")
        db_session.add(user)
        db_session.flush()
        result = pyramid_app.post(
            "/sign_in", params={"login": username, "password": password}
        ).json
        assert result["view"] == "bad_auth"

    def test_index_unlogged(self, db_session, pyramid_app):
        result = pyramid_app.post("/").json
        assert result["view"] == "index"
        assert result["username"] is None

    def test_index_logged(self, db_session, pyramid_app):
        user = add_user(db_session, user_name="username1", email="email")
        db_session.add(user)
        db_session.flush()
        result = pyramid_app.post(
            "/sign_in", params={"login": "username1", "password": "password"}
        )
        result = pyramid_app.post("/", headers=result.headers).json
        assert result["view"] == "index"
        assert result["username"] == "username1"



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import importlib
import logging

import pyramid.security

from ziggurat_foundations.models.base import get_db_session
from ziggurat_foundations.models.services.user import UserService

CONFIG_KEY = "ziggurat_foundations"
log = logging.getLogger(__name__)


class ZigguratSignInSuccess(object):
    def __contains__(self, other):
        return True

    def __init__(self, headers, came_from, user):
        self.headers = headers
        self.came_from = came_from
        self.user = user


class ZigguratSignInBadAuth(object):
    def __contains__(self, other):
        return False

    def __init__(self, headers, came_from):
        self.headers = headers
        self.came_from = came_from


class ZigguratSignOut(object):
    def __contains__(self, other):
        return True

    def __init__(self, headers):
        self.headers = headers


def includeme(config):
    settings = config.registry.settings
    sign_in_path = settings.get("%s.sign_in.sign_in_pattern" % CONFIG_KEY, "/sign_in")
    sign_out_path = settings.get(
        "%s.sign_in.sign_out_pattern" % CONFIG_KEY, "/sign_out"
    )
    session_provider_callable_config = settings.get(
        "%s.session_provider_callable" % CONFIG_KEY
    )
    signin_came_from_key = settings.get(
        "%s.sign_in.came_from_key" % CONFIG_KEY, "came_from"
    )
    signin_username_key = settings.get("%s.sign_in.username_key" % CONFIG_KEY, "login")
    signin_password_key = settings.get(
        "%s.sign_in.password_key" % CONFIG_KEY, "password"
    )

    if not session_provider_callable_config:

        def session_provider_callable(request):
            return get_db_session()

    else:
        if callable(session_provider_callable_config):
            session_provider_callable = session_provider_callable_config
        else:
            parts = session_provider_callable_config.split(":")
            _tmp = importlib.import_module(parts[0])
            session_provider_callable = getattr(_tmp, parts[1])

    endpoint = ZigguratSignInProvider(
        settings=settings,
        session_getter=session_provider_callable,
        signin_came_from_key=signin_came_from_key,
        signin_username_key=signin_username_key,
        signin_password_key=signin_password_key,
    )

    config.add_route(
        "ziggurat.routes.sign_in",
        sign_in_path,
        use_global_views=True,
        factory=endpoint.sign_in,
    )
    config.add_route(
        "ziggurat.routes.sign_out",
        sign_out_path,
        use_global_views=True,
        factory=endpoint.sign_out,
    )


class ZigguratSignInProvider(object):
    def __init__(self, *args, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def sign_in(self, request):
        came_from = request.params.get(self.signin_came_from_key, "/")
        db_session = self.session_getter(request)

        user = UserService.by_user_name(
            request.params.get(self.signin_username_key), db_session=db_session
        )
        if user is None:
            # if no result, test to see if email exists
            user = UserService.by_email(
                request.params.get(self.signin_username_key), db_session=db_session
            )
        if user:
            password = request.params.get(self.signin_password_key)
            if UserService.check_password(user, password):
                headers = pyramid.security.remember(request, user.id)
                return ZigguratSignInSuccess(
                    headers=headers, came_from=came_from, user=user
                )
        headers = pyramid.security.forget(request)
        return ZigguratSignInBadAuth(headers=headers, came_from=came_from)

    def sign_out(self, request):
        headers = pyramid.security.forget(request)
        return ZigguratSignOut(headers=headers)

    def session_getter(self, request):
        raise NotImplementedError()



"""add id/parent id to resource structure

Revision ID: 5c84d7260c5
Revises: 24ab8d11f014
Create Date: 2011-11-11 00:09:09.624704

"""
from __future__ import unicode_literals

import sqlalchemy as sa
from alembic import op

# downgrade revision identifier, used by Alembic.
revision = "5c84d7260c5"
down_revision = "24ab8d11f014"


def upgrade():
    op.add_column(
        "resources",
        sa.Column(
            "parent_id",
            sa.BigInteger(),
            sa.ForeignKey(
                "resources.resource_id", onupdate="CASCADE", ondelete="SET NULL"
            ),
        ),
    )


def downgrade():
    pass



from __future__ import with_statement, unicode_literals
import os
from alembic import context
from sqlalchemy import create_engine
from sqlalchemy.schema import MetaData
from logging.config import fileConfig

# pylint: disable=no-member

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
if config.config_file_name:
    fileConfig(config.config_file_name)

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata

target_metadata = MetaData(
    naming_convention={
        "ix": "ix_%(column_0_label)s",
        "uq": "uq_%(table_name)s_%(column_0_name)s",
        "ck": "ck_%(table_name)s_%(constraint_name)s",
        "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
        "pk": "pk_%(table_name)s",
    }
)


def get_url():
    url = os.getenv("DB_URL", "")
    if url == "":
        url = config.get_main_option("sqlalchemy.url")

    return url


# target_metadata = None

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.


def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = get_url()
    context.configure(
        url=url,
        version_table="alembic_ziggurat_foundations_version",
        transaction_per_migration=True,
    )

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    engine = create_engine(get_url())

    connection = engine.connect()
    context.configure(
        connection=connection,
        target_metadata=target_metadata,
        version_table="alembic_ziggurat_foundations_version",
        transaction_per_migration=True,
    )

    try:
        with context.begin_transaction():
            context.run_migrations()
    finally:
        connection.close()


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



"""change all linking keys from chars to id's

Revision ID: 20671b28c538
Revises: 4c10d97c509
Create Date: 2012-07-07 21:49:21.906150

"""
from __future__ import unicode_literals

import sqlalchemy as sa
from alembic import op
from alembic.context import get_context
from sqlalchemy.dialects.mysql.base import MySQLDialect
from sqlalchemy.engine.reflection import Inspector

# revision identifiers, used by Alembic.
revision = "20671b28c538"
down_revision = "4c10d97c509"


def upgrade():
    c = get_context()
    insp = Inspector.from_engine(c.connection.engine)

    # existing migration
    # pre naming convention keys
    groups_permissions_pkey = "groups_permissions_pkey"
    groups_pkey = "groups_pkey"
    groups_resources_permissions_pkey = "groups_resources_permissions_pkey"
    users_groups_pkey = "users_groups_pkey"
    users_permissions_pkey = "users_permissions_pkey"
    users_resources_permissions_pkey = "users_resources_permissions_pkey"

    # inspected keys
    groups_permissions_pkey = insp.get_pk_constraint("groups_permissions")["name"]
    groups_pkey = insp.get_pk_constraint("groups")["name"]
    groups_resources_permissions_pkey = insp.get_pk_constraint(
        "groups_resources_permissions"
    )["name"]
    users_groups_pkey = insp.get_pk_constraint("users_groups")["name"]
    users_permissions_pkey = insp.get_pk_constraint("users_permissions")["name"]
    users_resources_permissions_pkey = insp.get_pk_constraint(
        "users_resources_permissions"
    )["name"]

    op.drop_constraint("groups_pkey", "groups", type_="primary")

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column(
            "groups", sa.Column("id", sa.Integer, primary_key=True, autoincrement=False)
        )
        op.create_primary_key(groups_pkey, "groups", cols=["id"])
        op.alter_column(
            "groups",
            "id",
            type_=sa.Integer,
            existing_type=sa.Integer,
            autoincrement=True,
            existing_autoincrement=False,
            nullable=False,
        )
    else:
        op.add_column(
            "groups", sa.Column("id", sa.Integer, primary_key=True, autoincrement=True)
        )
        op.create_primary_key(groups_pkey, "groups", cols=["id"])

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        for t in ["groups_permissions", "groups_resources_permissions", "users_groups"]:
            for constraint in insp.get_foreign_keys(t):
                if constraint["referred_columns"] == ["group_name"]:
                    op.drop_constraint(constraint["name"], t, type_="foreignkey")

        for t in ["users_resources_permissions", "users_permissions", "users_groups"]:
            for constraint in insp.get_foreign_keys(t):
                if constraint["referred_columns"] == ["user_name"]:
                    op.drop_constraint(constraint["name"], t, type_="foreignkey")

        for constraint in insp.get_foreign_keys("resources"):
            if constraint["referred_columns"] in [["user_name"], ["group_name"]]:
                op.drop_constraint(constraint["name"], "resources", type_="foreignkey")

    op.add_column(
        "resources",
        sa.Column(
            "owner_user_id",
            sa.Integer(),
            sa.ForeignKey("users.id", onupdate="CASCADE", ondelete="SET NULL"),
        ),
    )
    op.add_column(
        "resources",
        sa.Column(
            "owner_group_id",
            sa.Integer(),
            sa.ForeignKey("groups.id", onupdate="CASCADE", ondelete="SET NULL"),
        ),
    )
    # update the data
    resources_table = sa.Table(
        "resources", sa.MetaData(), autoload=True, autoload_with=c.connection
    )
    users_table = sa.Table(
        "users", sa.MetaData(), autoload=True, autoload_with=c.connection
    )
    groups_table = sa.Table(
        "groups", sa.MetaData(), autoload=True, autoload_with=c.connection
    )
    stmt = (
        resources_table.update()
        .values(owner_user_id=users_table.c.id)
        .where(users_table.c.user_name == resources_table.c.owner_user_name)
    )
    op.execute(stmt)
    stmt = (
        resources_table.update()
        .values(owner_group_id=groups_table.c.id)
        .where(groups_table.c.group_name == resources_table.c.owner_group_name)
    )
    op.execute(stmt)

    # mysql is stupid as usual so we cant create FKEY and add PKEY later,
    # need to set PKEY first and then set FKEY
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column("groups_permissions", sa.Column("group_id", sa.Integer()))
    else:
        op.add_column(
            "groups_permissions",
            sa.Column(
                "group_id",
                sa.Integer(),
                sa.ForeignKey(
                    "groups.id", onupdate="CASCADE", ondelete="CASCADE"  # noqa  # noqa
                ),
            ),
        )  # noqa

    groups_permissions_table = sa.Table(
        "groups_permissions", sa.MetaData(), autoload=True, autoload_with=c.connection
    )
    stmt = (
        groups_permissions_table.update()
        .values(group_id=groups_table.c.id)
        .where(groups_table.c.group_name == groups_permissions_table.c.group_name)
    )
    op.execute(stmt)

    op.drop_constraint(groups_permissions_pkey, "groups_permissions", type_="primary")
    op.create_primary_key(
        groups_permissions_pkey, "groups_permissions", cols=["group_id", "perm_name"]
    )
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(
            None,
            "groups_permissions",
            "groups",
            remote_cols=["id"],
            local_cols=["group_id"],
            onupdate="CASCADE",
            ondelete="CASCADE",
        )

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column(
            "groups_resources_permissions", sa.Column("group_id", sa.Integer())
        )
    else:
        op.add_column(
            "groups_resources_permissions",
            sa.Column(
                "group_id",
                sa.Integer(),
                sa.ForeignKey("groups.id", onupdate="CASCADE", ondelete="CASCADE"),
            ),
        )

    groups_resources_permissions_table = sa.Table(
        "groups_resources_permissions",
        sa.MetaData(),
        autoload=True,
        autoload_with=c.connection,
    )
    stmt = (
        groups_resources_permissions_table.update()
        .values(group_id=groups_table.c.id)
        .where(
            groups_table.c.group_name == groups_resources_permissions_table.c.group_name
        )
    )
    op.execute(stmt)
    op.drop_constraint(
        groups_resources_permissions_pkey,
        "groups_resources_permissions",
        type_="primary",
    )
    op.create_primary_key(
        groups_resources_permissions_pkey,
        "groups_resources_permissions",
        cols=["group_id", "resource_id", "perm_name"],
    )

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(
            None,
            "groups_resources_permissions",
            "groups",
            remote_cols=["id"],
            local_cols=["group_id"],
            onupdate="CASCADE",
            ondelete="CASCADE",
        )

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column("users_groups", sa.Column("group_id", sa.Integer()))
    else:
        op.add_column(
            "users_groups",
            sa.Column(
                "group_id",
                sa.Integer(),
                sa.ForeignKey(
                    "groups.id", onupdate="CASCADE", ondelete="CASCADE"  # noqa
                ),
            ),
        )  # noqa

    users_groups_table = sa.Table(
        "users_groups", sa.MetaData(), autoload=True, autoload_with=c.connection
    )
    stmt = (
        users_groups_table.update()
        .values(group_id=groups_table.c.id)
        .where(groups_table.c.group_name == users_groups_table.c.group_name)
    )
    op.execute(stmt)

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column("users_groups", sa.Column("user_id", sa.Integer()))
    else:
        op.add_column(
            "users_groups",
            sa.Column(
                "user_id",
                sa.Integer(),
                sa.ForeignKey(
                    "users.id", onupdate="CASCADE", ondelete="CASCADE"  # noqa
                ),
            ),
        )  # noqa

    users_groups_table = sa.Table(
        "users_groups", sa.MetaData(), autoload=True, autoload_with=c.connection
    )
    stmt = (
        users_groups_table.update()
        .values(user_id=users_table.c.id)
        .where(users_table.c.user_name == users_groups_table.c.user_name)
    )
    op.execute(stmt)
    op.drop_constraint(users_groups_pkey, "users_groups", type="primary")
    op.create_primary_key(
        users_groups_pkey, "users_groups", cols=["user_id", "group_id"]
    )
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(
            None,
            "users_groups",
            "groups",
            remote_cols=["id"],
            local_cols=["group_id"],
            onupdate="CASCADE",
            ondelete="CASCADE",
        )
        op.create_foreign_key(
            None,
            "users_groups",
            "users",
            remote_cols=["id"],
            local_cols=["user_id"],
            onupdate="CASCADE",
            ondelete="CASCADE",
        )

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column("users_permissions", sa.Column("user_id", sa.Integer()))
    else:
        op.add_column(
            "users_permissions",
            sa.Column(
                "user_id",
                sa.Integer(),
                sa.ForeignKey(
                    "users.id", onupdate="CASCADE", ondelete="CASCADE"  # noqa
                ),
            ),
        )  # noqa

    users_permissions_table = sa.Table(
        "users_permissions", sa.MetaData(), autoload=True, autoload_with=c.connection
    )
    stmt = (
        users_permissions_table.update()
        .values(user_id=users_table.c.id)
        .where(users_table.c.user_name == users_permissions_table.c.user_name)
    )
    op.execute(stmt)
    op.drop_constraint(users_permissions_pkey, "users_permissions", type="primary")
    op.create_primary_key(
        users_permissions_pkey, "users_permissions", cols=["user_id", "perm_name"]
    )
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(
            None,
            "users_permissions",
            "users",
            remote_cols=["id"],
            local_cols=["user_id"],
            onupdate="CASCADE",
            ondelete="CASCADE",
        )

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column("users_resources_permissions", sa.Column("user_id", sa.Integer()))
    else:
        op.add_column(
            "users_resources_permissions",
            sa.Column(
                "user_id",
                sa.Integer(),
                sa.ForeignKey("users.id", onupdate="CASCADE", ondelete="CASCADE"),
            ),
        )

    users_resources_permissions_table = sa.Table(
        "users_resources_permissions",
        sa.MetaData(),
        autoload=True,
        autoload_with=c.connection,
    )
    stmt = (
        users_resources_permissions_table.update()
        .values(user_id=users_table.c.id)
        .where(users_table.c.user_name == users_resources_permissions_table.c.user_name)
    )
    op.execute(stmt)

    op.drop_constraint(
        users_resources_permissions_pkey, "users_resources_permissions", type="primary"
    )
    op.create_primary_key(
        users_resources_permissions_pkey,
        "users_resources_permissions",
        cols=["user_id", "resource_id", "perm_name"],
    )
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(
            None,
            "users_resources_permissions",
            "users",
            remote_cols=["id"],
            local_cols=["user_id"],
            onupdate="CASCADE",
            ondelete="CASCADE",
        )

    op.drop_column("resources", "owner_user_name")
    op.drop_column("resources", "owner_group_name")
    op.drop_column("groups_permissions", "group_name")
    op.drop_column("groups_resources_permissions", "group_name")
    op.drop_column("users_resources_permissions", "user_name")
    op.drop_column("users_groups", "group_name")
    op.drop_column("users_groups", "user_name")
    op.drop_column("users_permissions", "user_name")


def downgrade():
    pass



"""groups pkey change

Revision ID: 3cfc41c4a5f0
Revises: 53927300c277
Create Date: 2012-06-27 02:15:58.776223

"""
from __future__ import unicode_literals

import sqlalchemy as sa
from alembic import op
from alembic.context import get_context
from sqlalchemy.dialects.mysql.base import MySQLDialect
from sqlalchemy.engine.reflection import Inspector

# revision identifiers, used by Alembic.
revision = "3cfc41c4a5f0"
down_revision = "53927300c277"


def upgrade():
    c = get_context()
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        insp = Inspector.from_engine(c.connection.engine)
        for t in [
            "groups_permissions",
            "groups_resources_permissions",
            "users_groups",
            "resources",
        ]:
            for constraint in insp.get_foreign_keys(t):
                if constraint["referred_columns"] == ["group_name"]:
                    op.drop_constraint(constraint["name"], t, type="foreignkey")

    op.drop_column("groups", "id")
    op.alter_column(
        "groups", "group_name", type_=sa.Unicode(128), existing_type=sa.Unicode(50)
    )
    op.create_primary_key("groups_pkey", "groups", cols=["group_name"])

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(
            None,
            "groups_permissions",
            "groups",
            remote_cols=["group_name"],
            local_cols=["group_name"],
            onupdate="CASCADE",
            ondelete="CASCADE",
        )
        op.create_foreign_key(
            None,
            "groups_resources_permissions",
            "groups",
            remote_cols=["group_name"],
            local_cols=["group_name"],
            onupdate="CASCADE",
            ondelete="CASCADE",
        )
        op.create_foreign_key(
            None,
            "users_groups",
            "groups",
            remote_cols=["group_name"],
            local_cols=["group_name"],
            onupdate="CASCADE",
            ondelete="CASCADE",
        )
        op.create_foreign_key(
            None,
            "resources",
            "groups",
            remote_cols=["group_name"],
            local_cols=["owner_group_name"],
            onupdate="CASCADE",
            ondelete="SET NULL",
        )


def downgrade():
    pass



# -*- coding: utf-8 -*-
from __future__ import with_statement, unicode_literals

from ziggurat_foundations.models.base import get_db_session
from ziggurat_foundations.tests import add_user, BaseTestCase, DummyUserObj
from ziggurat_foundations.tests.conftest import User
from ziggurat_foundations.models.services.user import UserService


class TestModel(BaseTestCase):
    def test_get_keys(self, db_session):
        keys = User._get_keys()
        assert len(keys) == 9

    def test_get_dict(self, db_session):
        created_user = add_user(db_session)
        dict_ = created_user.get_dict()
        assert len(dict_) == 9

    def test_get_dict_excluded(self, db_session):
        created_user = add_user(db_session)
        dict_ = created_user.get_dict(exclude_keys=["user_name"])
        assert "user_name" not in dict_

    def test_get_dict_included(self, db_session):
        created_user = add_user(db_session)
        dict_ = created_user.get_dict(include_keys=["user_name"])
        assert ["user_name"] == list(dict_.keys())

    def test_get_dict_included_excluded(self, db_session):
        created_user = add_user(db_session)
        dict_ = created_user.get_dict(
            include_keys=["user_name", "id", "email", "status"], exclude_keys=["email"]
        )
        assert sorted(["user_name", "id", "status"]) == sorted(dict_.keys())

    def test_appstruct(self, db_session):
        created_user = add_user(db_session)
        appstruct = created_user.get_appstruct()
        assert len(appstruct) == 9

    def test_populate_obj_appstruct(self, db_session):
        created_user = add_user(db_session)
        # reset password
        created_user.user_password = None
        app_struct = {
            "user_name": "new_name",
            "user_password": "foo",
            "email": "change@email.com",
        }
        created_user.populate_obj(app_struct)
        assert created_user.user_name == app_struct["user_name"]
        assert created_user.user_password == app_struct["user_password"]
        assert created_user.email == app_struct["email"]

    def test_populate_obj_appstruct_exclude(self, db_session):
        created_user = add_user(db_session)
        # reset password
        created_user.user_password = None
        app_struct = {
            "user_name": "new_name",
            "user_password": "foo",
            "email": "change@email.com",
        }
        created_user.populate_obj(app_struct, exclude_keys=["user_password"])
        assert created_user.user_name == app_struct["user_name"]
        assert created_user.user_password is None
        assert created_user.email == app_struct["email"]

    def test_populate_obj_appstruct_include(self, db_session):
        created_user = add_user(db_session)
        # reset password
        created_user.user_password = None
        app_struct = {
            "user_name": "new_name",
            "user_password": "foo",
            "email": "change@email.com",
        }
        created_user.populate_obj(app_struct, include_keys=["user_password"])
        assert created_user.user_name != app_struct["user_name"]
        assert created_user.user_password == app_struct["user_password"]
        assert created_user.email != app_struct["email"]

    def test_populate_obj_obj(self, db_session):
        created_user = add_user(db_session)
        # reset password
        created_user.user_password = None
        test_obj = DummyUserObj()
        created_user.populate_obj_from_obj(test_obj)
        assert created_user.user_name == test_obj.user_name
        assert created_user.user_password == test_obj.user_password
        assert created_user.email == test_obj.email

    def test_populate_obj_obj_exclude(self, db_session):
        created_user = add_user(db_session)
        # reset password
        created_user.user_password = None
        test_obj = DummyUserObj()
        created_user.populate_obj_from_obj(test_obj, exclude_keys=["user_password"])
        assert created_user.user_name == test_obj.user_name
        assert created_user.user_password is None
        assert created_user.email == test_obj.email

    def test_populate_obj_obj_include(self, db_session):
        created_user = add_user(db_session)
        # reset password
        created_user.user_password = None
        test_obj = DummyUserObj()
        created_user.populate_obj_from_obj(test_obj, include_keys=["user_password"])
        assert created_user.user_name != test_obj.user_name
        assert created_user.user_password == test_obj.user_password
        assert created_user.email != test_obj.email

    def test_session(self, db_session):
        from sqlalchemy.orm.session import Session

        session = get_db_session(None, add_user(db_session))
        assert isinstance(session, Session)

    def test_add_object_without_flush(self, db_session):
        user = User(user_name="some_new_user", email="foo")
        assert user.id is None
        user.persist(db_session=db_session)
        assert user.id is None

    def test_add_object_with_flush(self, db_session):
        user = User(user_name="some_new_user", email="foo")
        assert user.id is None
        user.persist(flush=True, db_session=db_session)
        assert user.id is not None

    def test_delete_object_with_flush(self, db_session):
        user = User(user_name="some_new_user", email="foo")
        assert user.id is None
        user.persist(flush=True, db_session=db_session)
        assert user.id is not None
        uid = user.id
        UserService.by_id(uid, db_session=db_session) is not None
        user.delete()
        assert UserService.by_id(uid, db_session=db_session) is None


class TestMigrations(BaseTestCase):
    def test_migrations(self, db_session):
        pass



# -*- coding: utf-8 -*-

from collections import namedtuple

import sqlalchemy as sa

from .models.base import get_db_session

try:
    from pyramid.security import Allow, Deny, ALL_PERMISSIONS

except ImportError as e:
    Allow = "Allow"
    Deny = "Deny"

    # borrowed directly from pyramid - to avoid dependency on pyramid itself
    # source https://github.com/Pylons/pyramid/blob/master/pyramid/security.py

    class AllPermissionsList(object):

        """ Stand in 'permission list' to represent all permissions """

        def __iter__(self):
            return ()

        def __contains__(self, other):
            return True

        def __eq__(self, other):
            return isinstance(other, self.__class__)

    ALL_PERMISSIONS = AllPermissionsList()

__all__ = [
    "ANY_PERMISSION_CLS",
    "ANY_PERMISSION",
    "resource_permissions_for_users",
    "permission_to_04_acls",
    "permission_to_pyramid_acls",
]


class ANY_PERMISSION_CLS(object):
    def __eq__(self, other):
        return "__any_permission__" == other

    def __ne__(self, other):
        return "__any_permission__" != other


ANY_PERMISSION = ANY_PERMISSION_CLS()

PermissionTuple = namedtuple(
    "PermissionTuple",
    ["user", "perm_name", "type", "group", "resource", "owner", "allowed"],
)


def resource_permissions_for_users(
    models_proxy,
    perm_names,
    resource_ids=None,
    user_ids=None,
    group_ids=None,
    resource_types=None,
    limit_group_permissions=False,
    skip_user_perms=False,
    skip_group_perms=False,
    db_session=None,
):
    """
    Returns permission tuples that match one of passed permission names
    perm_names - list of permissions that can be matched
    user_ids - restrict to specific users
    group_ids - restrict to specific groups
    resource_ids - restrict to specific resources
    limit_group_permissions - should be used if we do not want to have
    user objects returned for group permissions, this might cause performance
    issues for big groups
    """
    db_session = get_db_session(db_session)

    # fetch groups and their permissions (possibly with users belonging
    # to group if needed)
    query = db_session.query(
        models_proxy.GroupResourcePermission.perm_name,
        models_proxy.User,
        models_proxy.Group,
        sa.literal("group").label("type"),
        models_proxy.Resource,
    )

    query = query.join(
        models_proxy.Group,
        models_proxy.Group.id == models_proxy.GroupResourcePermission.group_id,
    )
    query = query.join(
        models_proxy.Resource,
        models_proxy.Resource.resource_id
        == models_proxy.GroupResourcePermission.resource_id,
    )
    if limit_group_permissions:
        query = query.outerjoin(models_proxy.User, models_proxy.User.id == None)  # noqa
    else:
        query = query.join(
            models_proxy.UserGroup,
            models_proxy.UserGroup.group_id
            == models_proxy.GroupResourcePermission.group_id,
        )

        query = query.outerjoin(
            models_proxy.User, models_proxy.User.id == models_proxy.UserGroup.user_id
        )

    if resource_ids:
        query = query.filter(
            models_proxy.GroupResourcePermission.resource_id.in_(resource_ids)
        )
    if resource_types:
        query = query.filter(models_proxy.Resource.resource_type.in_(resource_types))

    if perm_names not in ([ANY_PERMISSION], ANY_PERMISSION) and perm_names:
        query = query.filter(
            models_proxy.GroupResourcePermission.perm_name.in_(perm_names)
        )
    if group_ids:
        query = query.filter(
            models_proxy.GroupResourcePermission.group_id.in_(group_ids)
        )

    if user_ids and not limit_group_permissions:
        query = query.filter(models_proxy.UserGroup.user_id.in_(user_ids))

    # 2nd query that will fetch users with direct resource permissions
    query2 = db_session.query(
        models_proxy.UserResourcePermission.perm_name,
        models_proxy.User,
        models_proxy.Group,
        sa.literal("user").label("type"),
        models_proxy.Resource,
    )
    query2 = query2.join(
        models_proxy.User,
        models_proxy.User.id == models_proxy.UserResourcePermission.user_id,
    )
    query2 = query2.join(
        models_proxy.Resource,
        models_proxy.Resource.resource_id
        == models_proxy.UserResourcePermission.resource_id,
    )

    # group needs to be present to work for union, but never actually matched
    query2 = query2.outerjoin(models_proxy.Group, models_proxy.Group.id == None)  # noqa
    if perm_names not in ([ANY_PERMISSION], ANY_PERMISSION) and perm_names:
        query2 = query2.filter(
            models_proxy.UserResourcePermission.perm_name.in_(perm_names)
        )
    if resource_ids:
        query2 = query2.filter(
            models_proxy.UserResourcePermission.resource_id.in_(resource_ids)
        )
    if resource_types:
        query2 = query2.filter(models_proxy.Resource.resource_type.in_(resource_types))
    if user_ids:
        query2 = query2.filter(
            models_proxy.UserResourcePermission.user_id.in_(user_ids)
        )

    if not skip_group_perms and not skip_user_perms:
        query = query.union(query2)
    elif skip_group_perms:
        query = query2

    users = [
        PermissionTuple(
            row.User,
            row.perm_name,
            row.type,
            row.Group or None,
            row.Resource,
            False,
            True,
        )
        for row in query
    ]
    return users


def permission_to_04_acls(permissions):
    """
    Legacy acl format kept for bw. compatibility
    :param permissions:
    :return:
    """
    acls = []
    for perm in permissions:
        if perm.type == "user":
            acls.append((perm.user.id, perm.perm_name))
        elif perm.type == "group":
            acls.append(("group:%s" % perm.group.id, perm.perm_name))
    return acls


def permission_to_pyramid_acls(permissions):
    """
    Returns a list of permissions in a format understood by pyramid
    :param permissions:
    :return:
    """
    acls = []
    for perm in permissions:
        if perm.type == "user":
            acls.append((Allow, perm.user.id, perm.perm_name))
        elif perm.type == "group":
            acls.append((Allow, "group:%s" % perm.group.id, perm.perm_name))
    return acls



# -*- coding: utf-8 -*-
from __future__ import with_statement, unicode_literals

from ziggurat_foundations.models.services.group import GroupService
from ziggurat_foundations.tests import add_user, add_group, BaseTestCase
from ziggurat_foundations.tests.conftest import Group


class TestGroup(BaseTestCase):
    def test_add_group(self, db_session):
        group = Group(group_name="example group", description="example description")
        db_session.add(group)
        db_session.flush()

        group = db_session.query(Group)
        group = group.filter(Group.group_name == "example group").first()

        assert group.group_name == "example group"
        assert group.description == "example description"
        assert group.member_count == 0

    def test_group_repr(self, db_session):
        group = add_group(db_session)
        assert repr(group) == "<Group: group, 1>"

    def test_by_group_name(self, db_session):
        added_group = add_group(db_session)
        queried_group = GroupService.by_group_name("group", db_session=db_session)

        assert added_group == queried_group

    def test_by_group_name_wrong_groupname(self, db_session):
        add_group(db_session)
        queried_group = GroupService.by_group_name(
            "not existing group", db_session=db_session
        )

        assert queried_group is None

    def test_users(self, db_session):
        user1 = add_user(db_session, "user1", "email1")
        user2 = add_user(db_session, "user2", "email2")

        group = add_group(db_session)
        group.users.append(user1)
        group.users.append(user2)

        assert group.users[0] == user1
        assert group.users[1] == user2

    def test_users_dynamic(self, db_session):
        user1 = add_user(db_session, "user1", "email1")
        user2 = add_user(db_session, "user2", "email2")

        group = add_group(db_session)
        group.users.append(user1)
        group.users.append(user2)
        group_users = group.users_dynamic.all()

        assert group_users[0] == user1
        assert group_users[1] == user2

    def test_get(self, db_session):
        group1 = add_group(db_session, "group1")
        group = GroupService.get(group_id=group1.id, db_session=db_session)
        assert group.id == group1.id

    def test_all(self, db_session):
        group1 = add_group(db_session, "group1")
        group2 = add_group(db_session, "group2")

        all_groups = Group.all(db_session=db_session).all()

        assert len(all_groups) == 2
        assert all_groups[0] == group1
        assert all_groups[1] == group2

    def test_user_paginator(self, db_session):
        user1 = add_user(db_session, "user1", "email1")
        user2 = add_user(db_session, "user2", "email2")

        group = add_group(db_session)
        group.users.append(user1)
        group.users.append(user2)
        users_count = len(group.users)
        get_params = {"foo": "bar", "baz": "xxx"}

        paginator = GroupService.get_user_paginator(
            group, 1, users_count, GET_params=get_params
        )

        assert paginator.page == 1
        assert paginator.first_item == 1
        assert paginator.last_item == 2
        assert paginator.items == [user1, user2]
        assert paginator.items == [user1, user2]
        assert paginator.items == [user1, user2]
        assert paginator.items == [user1, user2]

    def test_user_paginator_usernames(self, db_session):
        user1 = add_user(db_session, "user1", "email1")
        user2 = add_user(db_session, "user2", "email2")
        user3 = add_user(db_session, "user3", "email3")

        group = add_group(db_session)
        group.users.append(user1)
        group.users.append(user2)
        group.users.append(user3)

        # TODO: users count when filtering on names?
        paginator = GroupService.get_user_paginator(group, 1, user_ids=[1, 3])

        assert paginator.page == 1
        assert paginator.first_item == 1
        assert paginator.last_item == 2
        assert paginator.items == [user1, user3]
        assert paginator.item_count == 2
        assert paginator.page_count == 1



# -*- coding: utf-8 -*-
from __future__ import with_statement, unicode_literals

from ziggurat_foundations.models.services.external_identity import (
    ExternalIdentityService,
)
from ziggurat_foundations.tests import add_user, add_resource, BaseTestCase
from ziggurat_foundations.tests.conftest import (
    UserGroup,
    ExternalIdentity,
    GroupResourcePermission,
)


class TestUserGroup(BaseTestCase):
    def test_repr(self, db_session):
        user_group = UserGroup(user_id=1, group_id=1)

        assert repr(user_group) == "<UserGroup: g:1, u:1>"


class TestGroupResourcePermission(BaseTestCase):
    def test_repr(self, db_session):
        group_resource_perm = GroupResourcePermission(
            group_id=1, resource_id=1, perm_name="perm"
        )
        assert repr(group_resource_perm) == "<GroupResourcePermission: g:1, perm, r:1>"


class TestAddResource(BaseTestCase):
    def test_pkey(self, db_session):
        resource = add_resource(db_session, 99, "some random name")
        assert resource.resource_id == 99

    def test_nopkey(self, db_session):
        resource = add_resource(db_session, None, "some random name")
        assert resource.resource_id == 1


class TestExternalIdentity(BaseTestCase):
    def test_by_external_id_and_provider(self, db_session):
        user = add_user(db_session)
        identity = ExternalIdentity(
            external_user_name="foo", external_id="foo", provider_name="facebook"
        )
        user.external_identities.append(identity)
        # db_session.flush()
        found = ExternalIdentityService.by_external_id_and_provider(
            provider_name="facebook", external_id="foo", db_session=db_session
        )
        assert identity == found

    def test_get(self, db_session):
        user = add_user(db_session)
        identity = ExternalIdentity(
            external_user_name="foo", external_id="foo", provider_name="facebook"
        )
        user.external_identities.append(identity)
        db_session.flush()
        found = ExternalIdentityService.get(
            provider_name="facebook",
            external_id="foo",
            local_user_id=user.id,
            db_session=db_session,
        )
        assert identity == found

    def test_user_by_external_id_and_provider(self, db_session):
        user = add_user(db_session)
        identity = ExternalIdentity(
            external_user_name="foo", external_id="foo", provider_name="facebook"
        )
        user.external_identities.append(identity)
        # db_session.flush()
        found = ExternalIdentityService.user_by_external_id_and_provider(
            provider_name="facebook", external_id="foo", db_session=db_session
        )
        assert user == found


class TestUtils(BaseTestCase):
    def test_permission_to_04_acls(self, db_session):
        pass



"""add index to resources

Revision ID: 4c10d97c509
Revises: 3cfc41c4a5f0
Create Date: 2012-06-27 03:09:12.392704

"""
from __future__ import unicode_literals

from alembic import op

# revision identifiers, used by Alembic.
revision = "4c10d97c509"
down_revision = "3cfc41c4a5f0"


def upgrade():
    op.create_index("owner_user_name_ix", "resources", ["owner_user_name"])
    op.create_index("owner_group_name_ix", "resources", ["owner_group_name"])


def downgrade():
    pass



"""create ordering column

Revision ID: 264049f80948
Revises: 46a9c4fb9560
Create Date: 2012-02-13 20:32:34.542829

"""
from __future__ import unicode_literals

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "264049f80948"
down_revision = "46a9c4fb9560"


def upgrade():
    op.add_column(
        "resources",
        sa.Column("ordering", sa.Integer, server_default="0", nullable=False),
    )


def downgrade():
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sqlalchemy as sa
from zope.deprecation import deprecation

from ziggurat_foundations.exc import ZigguratSessionException


class BaseModel(object):
    """ Basic class that all other classes inherit from that supplies some
    basic methods useful for interaction with packages like:
    deform, colander or wtforms """

    @classmethod
    def _get_keys(cls):
        """ returns column names for this model """
        return sa.orm.class_mapper(cls).c.keys()

    @classmethod
    def get_primary_key(cls):
        return sa.orm.class_mapper(cls).primary_key

    def get_dict(self, exclude_keys=None, include_keys=None):
        """
        return dictionary of keys and values corresponding to this model's
        data - if include_keys is null the function will return all keys

        :param exclude_keys: (optional) is a list of columns from model that
        should not be returned by this function
        :param include_keys: (optional) is a list of columns from model that
        should be returned by this function
        :return:
        """
        d = {}
        exclude_keys_list = exclude_keys or []
        include_keys_list = include_keys or []
        for k in self._get_keys():
            if k not in exclude_keys_list and (
                k in include_keys_list or not include_keys
            ):
                d[k] = getattr(self, k)
        return d

    def get_appstruct(self):
        """ return list of tuples keys and values corresponding to this model's
        data """
        result = []
        for k in self._get_keys():
            result.append((k, getattr(self, k)))
        return result

    def populate_obj(self, appstruct, exclude_keys=None, include_keys=None):
        """
        updates instance properties *for column names that exist*
        for this model and are keys present in passed dictionary

        :param appstruct: (dictionary)
        :param exclude_keys: (optional) is a list of columns from model that
        should not be updated by this function
        :param include_keys: (optional) is a list of columns from model that
        should be updated by this function
        :return:
        """
        exclude_keys_list = exclude_keys or []
        include_keys_list = include_keys or []
        for k in self._get_keys():
            if (
                k in appstruct
                and k not in exclude_keys_list
                and (k in include_keys_list or not include_keys)
            ):
                setattr(self, k, appstruct[k])

    def populate_obj_from_obj(self, instance, exclude_keys=None, include_keys=None):
        """
        updates instance properties *for column names that exist*
        for this model and are properties present in passed dictionary

        :param instance:
        :param exclude_keys: (optional) is a list of columns from model that
        should not be updated by this function
        :param include_keys: (optional) is a list of columns from model that
        should be updated by this function
        :return:
        """
        exclude_keys_list = exclude_keys or []
        include_keys_list = include_keys or []
        for k in self._get_keys():
            if (
                hasattr(instance, k)
                and k not in exclude_keys_list
                and (k in include_keys_list or not include_keys)
            ):
                setattr(self, k, getattr(instance, k))

    def get_db_session(self, session=None):
        """
        Attempts to return session via get_db_session utility function
        :meth:`~ziggurat_foundations.models.get_db_session`

        :param session:
        :return:
        """
        return get_db_session(session, self)

    def persist(self, flush=False, db_session=None):
        """

        Adds object to session, if the object was freshly created this will
        persist the object in the storage on commit

        :param flush: boolean - if true then the session will be flushed
            instantly
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        db_session.add(self)
        if flush:
            db_session.flush()

    def delete(self, db_session=None):
        """
        Deletes the object via session, this will permanently delete the
        object from storage on commit

        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, self)
        db_session.delete(self)

    @classmethod
    @deprecation.deprecate(
        "BaseModel.all " "will be removed in 0.8, use service instead"
    )
    def base_query(cls, db_session=None):
        """
        Returns a base query object one can use to search on simple properties

        :param db_session:
        :return:
        """
        return get_db_session(db_session).query(cls)

    @classmethod
    @deprecation.deprecate(
        "BaseModel.all " "will be removed in 0.8, use service instead"
    )
    def all(cls, db_session=None):
        """
        Alias for base_query()

        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return cls.base_query(db_session)


def get_db_session(session=None, obj=None):
    """
    utility function that attempts to return sqlalchemy session that could
    have been created/passed in one of few ways:

    * It first tries to read session attached to instance
      if object argument was passed

    * then it tries to return  session passed as argument

    * finally tries to read pylons-like threadlocal called DBSession

    * if this fails exception is thrown

    :param session:
    :param obj:
    :return:
    """
    # try to read the session from instance
    from ziggurat_foundations import models

    if obj:
        return sa.orm.session.object_session(obj)
    # try passed session
    elif session:
        return session
    # try global pylons-like session then
    elif models.DBSession:
        return models.DBSession
    raise ZigguratSessionException("No Session found")



# -*- coding: utf-8 -*-


class ZigguratException(Exception):
    def __init__(self, msg, value=None):
        self.msg = msg
        self.value = value

    def __str__(self):
        return self.msg.format(self.value)


class ZigguratSessionException(ZigguratException):
    pass


class ZugguratTreeException(ZigguratException):
    pass


class ZigguratResourceTreeMissingException(ZugguratTreeException):
    pass


class ZigguratResourceTreePathException(ZugguratTreeException):
    pass


class ZigguratResourceOutOfBoundaryException(ZugguratTreeException):
    pass



# -*- coding: utf-8 -*-
from __future__ import with_statement, unicode_literals

import pytest

from ziggurat_foundations.models.services.group_permission import GroupPermissionService
from ziggurat_foundations.models.services.group_resource_permission import (
    GroupResourcePermissionService,
)
from ziggurat_foundations.models.services.user_permission import UserPermissionService
from ziggurat_foundations.models.services.user_resource_permission import (
    UserResourcePermissionService,
)

from ziggurat_foundations.models.services.resource import ResourceService

from ziggurat_foundations.permissions import PermissionTuple, ALL_PERMISSIONS
from ziggurat_foundations.tests import (
    add_user,
    check_one_in_other,
    add_resource,
    add_resource_b,
    add_group,
    BaseTestCase,
)
from ziggurat_foundations.tests.conftest import (
    User,
    UserPermission,
    GroupPermission,
    UserResourcePermission,
    GroupResourcePermission,
    TestResourceB,
)
from ziggurat_foundations.models.services.group import GroupService
from ziggurat_foundations.models.services.user import UserService


class TestUserPermissions(BaseTestCase):
    def test_user_permissions(self, db_session):
        created_user = add_user(db_session)
        permissions = UserService.permissions(created_user, db_session=db_session)
        expected = [
            PermissionTuple(
                created_user, "alter_users", "user", None, None, False, True
            ),
            PermissionTuple(created_user, "root", "user", None, None, False, True),
        ]
        check_one_in_other(permissions, expected)

    def test_owned_permissions(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, "test_resource")
        created_user.resources.append(resource)
        db_session.flush()
        resources = UserService.resources_with_perms(
            created_user, ["test_perm"], db_session=db_session
        ).all()
        assert resources[0] == resource

    def test_resources_with_perm(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, "test_resource")
        permission = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource.resource_id,
        )
        resource.user_permissions.append(permission)
        db_session.flush()
        resources = UserService.resources_with_perms(
            created_user, ["test_perm"], db_session=db_session
        ).all()
        assert resources[0] == resource

    def test_mixed_perms(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, "test_resource")
        permission = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource.resource_id,
        )
        resource.user_permissions.append(permission)
        resource2 = add_resource(db_session, 2, "test_resource")
        created_user.resources.append(resource2)
        add_resource(db_session, 3, "test_resource")
        add_resource_b(db_session, 4, "test_resource")
        db_session.flush()
        resources = UserService.resources_with_perms(
            created_user, ["test_perm"], db_session=db_session
        ).all()
        found_ids = [r.resource_id for r in resources]
        assert sorted(found_ids) == [1, 2]

    def test_resources_with_perm_type_found(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, "test_resource")
        permission = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource.resource_id,
        )
        resource.user_permissions.append(permission)
        db_session.flush()
        resources = UserService.resources_with_perms(
            created_user,
            ["test_perm"],
            resource_types=["test_resource"],
            db_session=db_session,
        ).all()
        assert resources[0] == resource

    def test_resources_with_perm_type_not_found(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, "test_resource")
        permission = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource.resource_id,
        )
        resource.user_permissions.append(permission)
        db_session.flush()
        resources = UserService.resources_with_perms(
            created_user,
            ["test_perm"],
            resource_types=["test_resource_b"],
            db_session=db_session,
        ).all()
        assert resources == []

    def test_resources_with_perm_type_other_found(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, "test_resource")
        resource2 = add_resource_b(db_session, 2, "test_resource")
        resource3 = add_resource(db_session, 3, "test_resource")
        resource4 = add_resource_b(db_session, 4, "test_resource")
        db_session.flush()
        permission = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource.resource_id,
        )
        resource.user_permissions.append(permission)
        permission2 = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource2.resource_id,
        )
        resource2.user_permissions.append(permission2)
        permission3 = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource3.resource_id,
        )
        resource3.user_permissions.append(permission3)
        permission4 = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource4.resource_id,
        )
        resource4.user_permissions.append(permission4)
        db_session.flush()
        resources = UserService.resources_with_perms(
            created_user,
            ["test_perm"],
            resource_types=["test_resource_b"],
            db_session=db_session,
        ).all()
        assert len(resources) == 2

    def test_resources_with_wrong_perm(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, "test_resource")
        permission = UserResourcePermission(
            perm_name="test_perm_bad",
            user_id=created_user.id,
            resource_id=resource.resource_id,
        )
        with pytest.raises(AssertionError):
            resource.user_permissions.append(permission)

    def test_multiple_resources_with_perm(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, "test_resource")
        permission = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource.resource_id,
        )
        resource.user_permissions.append(permission)
        resource2 = add_resource(db_session, 2, "test_resource2")
        permission2 = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource2.resource_id,
        )
        resource2.user_permissions.append(permission2)
        resources = UserService.resources_with_perms(
            created_user, ["test_perm"], db_session=db_session
        ).all()
        assert resources == [resource, resource2]

    def test_resources_ids_with_perm(self, db_session):
        created_user = add_user(db_session)
        resource1 = add_resource(db_session, 1, "test_resource1")
        resource2 = add_resource(db_session, 2, "test_resource2")
        resource3 = add_resource(db_session, 3, "test_resource3")

        permission1 = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource1.resource_id,
        )
        permission2 = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource2.resource_id,
        )
        permission3 = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource3.resource_id,
        )

        resource1.user_permissions.append(permission1)
        resource2.user_permissions.append(permission2)
        resource3.user_permissions.append(permission3)

        db_session.flush()
        resources = UserService.resources_with_perms(
            created_user, ["test_perm"], resource_ids=[1, 3], db_session=db_session
        ).all()
        assert resources == [resource1, resource3]

    def test_resources_with_wrong_group_permission(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, "test_resource")
        group = add_group(db_session)
        group.users.append(created_user)
        group_permission = GroupResourcePermission(
            perm_name="test_perm_bad",
            group_id=group.id,
            resource_id=resource.resource_id,
        )
        with pytest.raises(AssertionError):
            resource.group_permissions.append(group_permission)

    def test_resources_with_group_permission(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, "test_resource")
        resource2 = add_resource(db_session, 2, "test_resource2")
        add_resource(db_session, 3, "test_resource3")
        group = add_group(db_session)
        group.users.append(created_user)
        group_permission = GroupResourcePermission(
            perm_name="test_perm", group_id=1, resource_id=resource.resource_id
        )
        group_permission2 = GroupResourcePermission(
            perm_name="foo_perm", group_id=1, resource_id=resource2.resource_id
        )
        resource.group_permissions.append(group_permission)
        resource2.group_permissions.append(group_permission2)
        db_session.flush()
        resources = UserService.resources_with_perms(
            created_user, ["foo_perm"], db_session=db_session
        ).all()
        assert resources[0] == resource2

    def test_resources_with_direct_user_perms(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        # test_perm1 from group perms should be ignored
        perms = ResourceService.direct_perms_for_user(
            self.resource, self.user, db_session=db_session
        )
        second = [
            PermissionTuple(
                self.user, "foo_perm", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "test_perm2", "user", None, self.resource, False, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_resources_with_direct_group_perms(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        # test_perm1 from group perms should be ignored
        perms = ResourceService.group_perms_for_user(
            self.resource, self.user, db_session=db_session
        )
        second = [
            PermissionTuple(
                self.user, "group_perm", "group", self.group, self.resource, False, True
            )
        ]

        check_one_in_other(perms, second)

    def test_resources_with_user_perms(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        perms = ResourceService.perms_for_user(
            self.resource, self.user, db_session=db_session
        )
        second = [
            PermissionTuple(
                self.user, "foo_perm", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "test_perm2", "user", None, self.resource, False, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_for_perm(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        perms = ResourceService.users_for_perm(
            self.resource, "foo_perm", db_session=db_session
        )
        second = [
            PermissionTuple(
                self.user, "foo_perm", "user", None, self.resource, False, True
            )
        ]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm(self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        perms = ResourceService.users_for_perm(
            self.resource, "__any_permission__", db_session=db_session
        )
        second = [
            PermissionTuple(
                self.user, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "test_perm2", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "foo_perm", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user4,
                "group_perm",
                "group",
                self.group2,
                self.resource,
                False,
                True,
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm_resource_2(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        perms = ResourceService.users_for_perm(
            self.resource2, "__any_permission__", db_session=db_session
        )
        second = [
            PermissionTuple(
                self.user2, "foo_perm", "user", None, self.resource2, False, True
            ),
            PermissionTuple(
                self.user3, "test_perm", "user", None, self.resource2, False, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_limited_users(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        perms = ResourceService.users_for_perm(
            self.resource,
            "__any_permission__",
            user_ids=[self.user.id],
            db_session=db_session,
        )
        second = [
            PermissionTuple(
                self.user, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "test_perm2", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "foo_perm", "user", None, self.resource, False, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_limited_group(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        perms = ResourceService.users_for_perm(
            self.resource,
            "__any_permission__",
            user_ids=[self.user.id],
            group_ids=[self.group2.id],
            db_session=db_session,
        )
        second = [
            PermissionTuple(
                self.user, "test_perm2", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "foo_perm", "user", None, self.resource, False, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_limited_group_other_user_3(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        perms = ResourceService.users_for_perm(
            self.resource2,
            "__any_permission__",
            user_ids=[self.user3.id],
            db_session=db_session,
        )
        second = [
            PermissionTuple(
                self.user3, "test_perm", "user", None, self.resource2, False, True
            )
        ]

        check_one_in_other(perms, second)

    def test_resource_users_limited_group_other_user_4(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        perms = ResourceService.users_for_perm(
            self.resource,
            "__any_permission__",
            user_ids=[self.user4.id],
            group_ids=[self.group2.id],
            db_session=db_session,
        )
        second = [
            PermissionTuple(
                self.user4,
                "group_perm",
                "group",
                self.group2,
                self.resource,
                False,
                True,
            )
        ]

        check_one_in_other(perms, second)

    def test_resource_users_limited_group_ownage(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        resource = TestResourceB(
            resource_id=99, resource_name="other", owner_user_id=self.user2.id
        )
        group3 = add_group(db_session, "group 3")
        user2_permission = UserResourcePermission(
            perm_name="foo_perm", user_id=self.user2.id
        )
        group3_permission = GroupResourcePermission(
            perm_name="group_perm", group_id=group3.id
        )
        resource.group_permissions.append(group3_permission)
        resource.user_permissions.append(user2_permission)
        group3.users.append(self.user3)
        self.user.resources.append(resource)
        self.group2.resources.append(resource)
        db_session.flush()
        perms = ResourceService.users_for_perm(
            resource, "__any_permission__", db_session=db_session
        )
        second = [
            PermissionTuple(
                self.user2, "foo_perm", "user", None, resource, False, True
            ),
            PermissionTuple(
                self.user, ALL_PERMISSIONS, "user", None, resource, True, True
            ),
            PermissionTuple(
                self.user4, ALL_PERMISSIONS, "group", self.group2, resource, True, True
            ),
            PermissionTuple(
                self.user3, "group_perm", "group", group3, resource, False, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_users_for_perms(self, db_session):
        user = User(user_name="aaa", email="aaa", status=0)
        UserService.set_password(user, "password")
        aaa_perm = UserPermission(perm_name="aaa")
        bbb_perm = UserPermission(perm_name="bbb")
        bbb2_perm = UserPermission(perm_name="bbb")
        user.user_permissions.append(aaa_perm)
        user.user_permissions.append(bbb_perm)
        user2 = User(user_name="bbb", email="bbb", status=0)
        UserService.set_password(user2, "password")
        user2.user_permissions.append(bbb2_perm)
        user3 = User(user_name="ccc", email="ccc", status=0)
        UserService.set_password(user3, "password")
        group = add_group(db_session)
        group.users.append(user3)
        db_session.add(user)
        db_session.add(user2)
        db_session.flush()
        users = UserService.users_for_perms(["aaa"], db_session=db_session)
        assert len(users.all()) == 1
        assert users[0].user_name == "aaa"
        users = UserService.users_for_perms(["bbb"], db_session=db_session).all()
        assert len(users) == 2
        assert ["aaa", "bbb"] == sorted([u.user_name for u in users])
        users = UserService.users_for_perms(
            ["aaa", "bbb", "manage_apps"], db_session=db_session
        )
        assert ["aaa", "bbb", "ccc"] == sorted([u.user_name for u in users])

    def test_resources_with_possible_perms(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        resource = TestResourceB(
            resource_id=3, resource_name="other", owner_user_id=self.user.id
        )
        self.user.resources.append(resource)
        resource_g = TestResourceB(resource_id=4, resource_name="group owned")
        self.group.resources.append(resource_g)
        db_session.flush()
        perms = UserService.resources_with_possible_perms(
            self.user, db_session=db_session
        )
        second = [
            PermissionTuple(
                self.user, "foo_perm", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "test_perm2", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user, ALL_PERMISSIONS, "user", None, resource, True, True
            ),
            PermissionTuple(
                self.user, ALL_PERMISSIONS, "group", self.group, resource_g, True, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm_additional_users(self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, "user 6")
        user7 = add_user(db_session, 7, "user 7")
        perm2 = GroupResourcePermission(
            perm_name="group_perm2", resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        perms = ResourceService.users_for_perm(
            self.resource, "__any_permission__", db_session=db_session
        )
        second = [
            PermissionTuple(
                self.user, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                user6, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                user7, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                self.user,
                "group_perm2",
                "group",
                self.group,
                self.resource,
                False,
                True,
            ),
            PermissionTuple(
                user6, "group_perm2", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                user7, "group_perm2", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "test_perm2", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "foo_perm", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user4,
                "group_perm",
                "group",
                self.group2,
                self.resource,
                False,
                True,
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm_limited_group_perms(self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, "user 6")
        user7 = add_user(db_session, 7, "user 7")
        perm2 = GroupResourcePermission(
            perm_name="group_perm2", resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        perms = ResourceService.users_for_perm(
            self.resource,
            "__any_permission__",
            limit_group_permissions=True,
            db_session=db_session,
        )
        second = [
            PermissionTuple(
                None, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                None, "group_perm2", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "test_perm2", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "foo_perm", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                None, "group_perm", "group", self.group2, self.resource, False, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_groups_for_any_perm_additional_users(self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, "user 6")
        user7 = add_user(db_session, 7, "user 7")
        perm2 = GroupResourcePermission(
            perm_name="group_perm2", resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        perms = ResourceService.groups_for_perm(
            self.resource, "__any_permission__", db_session=db_session
        )
        second = [
            PermissionTuple(
                self.user, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                user6, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                user7, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                self.user,
                "group_perm2",
                "group",
                self.group,
                self.resource,
                False,
                True,
            ),
            PermissionTuple(
                user6, "group_perm2", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                user7, "group_perm2", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                self.user4,
                "group_perm",
                "group",
                self.group2,
                self.resource,
                False,
                True,
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_groups_for_any_perm_just_group_perms_limited(self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, "user 6")
        user7 = add_user(db_session, 7, "user 7")
        perm2 = GroupResourcePermission(
            perm_name="group_perm2", resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        perms = ResourceService.groups_for_perm(
            self.resource,
            "__any_permission__",
            limit_group_permissions=True,
            db_session=db_session,
        )
        second = [
            PermissionTuple(
                None, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                None, "group_perm2", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                None, "group_perm", "group", self.group2, self.resource, False, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm_excluding_group_perms(self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, "user 6")
        user7 = add_user(db_session, 7, "user 7")
        perm2 = GroupResourcePermission(
            perm_name="group_perm2", resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        perms = ResourceService.users_for_perm(
            self.resource,
            "__any_permission__",
            limit_group_permissions=True,
            skip_group_perms=True,
            db_session=db_session,
        )
        second = [
            PermissionTuple(
                self.user, "test_perm2", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "foo_perm", "user", None, self.resource, False, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_groups_for_any_perm_just_group_perms_limited_empty_group(
        self, db_session
    ):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, "user 6")
        user7 = add_user(db_session, 7, "user 7")
        perm2 = GroupResourcePermission(
            perm_name="group_perm2", resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)

        group3 = add_group(db_session, "Empty group")
        perm3 = GroupResourcePermission(
            perm_name="group_permx", resource_id=self.resource.resource_id
        )
        group3.resource_permissions.append(perm3)
        perms = ResourceService.groups_for_perm(
            self.resource,
            "__any_permission__",
            limit_group_permissions=True,
            db_session=db_session,
        )

        second = [
            PermissionTuple(
                None, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                None, "group_perm2", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                None, "group_perm", "group", self.group2, self.resource, False, True
            ),
            PermissionTuple(
                None, "group_permx", "group", group3, self.resource, False, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm_limited_group_perms_empty_group(
        self, db_session
    ):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, "user 6")
        user7 = add_user(db_session, 7, "user 7")
        perm2 = GroupResourcePermission(
            perm_name="group_perm2", resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        group3 = add_group(db_session, "Empty group")
        perm3 = GroupResourcePermission(
            perm_name="group_permx", resource_id=self.resource.resource_id
        )
        group3.resource_permissions.append(perm3)

        perms = ResourceService.users_for_perm(
            self.resource,
            "__any_permission__",
            limit_group_permissions=True,
            db_session=db_session,
        )

        second = [
            PermissionTuple(
                None, "group_perm", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                None, "group_perm2", "group", self.group, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "test_perm2", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                self.user, "foo_perm", "user", None, self.resource, False, True
            ),
            PermissionTuple(
                None, "group_perm", "group", self.group2, self.resource, False, True
            ),
            PermissionTuple(
                None, "group_permx", "group", group3, self.resource, False, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_get_resource_permission(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, "test_resource")
        permission = UserResourcePermission(
            perm_name="test_perm",
            user_id=created_user.id,
            resource_id=resource.resource_id,
        )
        resource.user_permissions.append(permission)
        db_session.flush()
        perm = UserResourcePermissionService.get(
            user_id=created_user.id,
            resource_id=resource.resource_id,
            perm_name="test_perm",
            db_session=db_session,
        )
        assert perm.perm_name == "test_perm"
        assert perm.resource_id == resource.resource_id
        assert perm.user_id == created_user.id


class TestGroupPermission(BaseTestCase):
    def test_repr(self, db_session):
        group_permission = GroupPermission(group_id=1, perm_name="perm")
        assert repr(group_permission) == "<GroupPermission: perm>"

    def test_get(self, db_session):
        org_group = add_group(db_session, "group1")
        group = GroupPermissionService.get(
            group_id=org_group.id, perm_name="manage_apps", db_session=db_session
        )
        assert group.group_id == 1
        assert group.perm_name == "manage_apps"

    def test_by_group_and_perm(self, db_session):
        add_group(db_session)
        queried = GroupPermissionService.by_group_and_perm(
            1, "manage_apps", db_session=db_session
        )
        assert queried.group_id == 1
        assert queried.perm_name == "manage_apps"

    def test_by_group_and_perm_wrong_group(self, db_session):
        add_group(db_session)
        queried = GroupPermissionService.by_group_and_perm(
            2, "manage_apps", db_session=db_session
        )
        assert queried is None

    def test_by_group_and_perm_wrong_perm(self, db_session):
        add_group(db_session)
        queried = GroupPermissionService.by_group_and_perm(
            1, "wrong_perm", db_session=db_session
        )
        assert queried is None

    def test_resources_with_possible_perms(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        perms = GroupService.resources_with_possible_perms(self.group)
        second = [
            PermissionTuple(
                None, "group_perm", "group", self.group, self.resource, False, True
            )
        ]

        check_one_in_other(perms, second)

    def test_resources_with_possible_perms_group2(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        resource3 = add_resource_b(db_session, 3, "other resource")
        self.group2.resources.append(resource3)
        group_permission2 = GroupResourcePermission(
            perm_name="group_perm2", group_id=self.group2.id
        )
        self.resource2.group_permissions.append(group_permission2)

        perms = GroupService.resources_with_possible_perms(self.group2)
        second = [
            PermissionTuple(
                None, "group_perm", "group", self.group2, self.resource, False, True
            ),
            PermissionTuple(
                None, "group_perm2", "group", self.group2, self.resource2, False, True
            ),
            PermissionTuple(
                None, ALL_PERMISSIONS, "group", self.group2, resource3, True, True
            ),
        ]

        check_one_in_other(perms, second)

    def test_group_resource_permission(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        add_resource_b(db_session, 3, "other resource")
        db_session.flush()
        group_permission2 = GroupResourcePermission(
            perm_name="group_perm2", group_id=self.group2.id
        )
        row = GroupResourcePermissionService.get(
            group_id=self.group2.id,
            resource_id=self.resource2.resource_id,
            perm_name="group_perm2",
            db_session=db_session,
        )
        assert row is None
        self.resource2.group_permissions.append(group_permission2)
        row = GroupResourcePermissionService.get(
            group_id=self.group2.id,
            resource_id=self.resource2.resource_id,
            perm_name="group_perm2",
            db_session=db_session,
        )
        assert row is not None

    def test_group_resource_permission_wrong(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        perm_name = "group_permX"
        perm = ResourceService.perm_by_group_and_perm_name(
            resource_id=self.resource.resource_id,
            group_id=self.group.id,
            perm_name=perm_name,
            db_session=db_session,
        )
        assert perm is None

    def test_group_resource_permission2(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        perm_name = "group_perm"
        perm = ResourceService.perm_by_group_and_perm_name(
            resource_id=self.resource.resource_id,
            group_id=self.group.id,
            perm_name=perm_name,
            db_session=db_session,
        )
        assert perm.group_id == self.group.id
        assert perm.resource_id == self.resource.resource_id
        assert perm.perm_name == perm_name


class TestUserPermission(BaseTestCase):
    def test_repr(self, db_session):
        user_permission = UserPermission(user_id=1, perm_name="perm")
        assert repr(user_permission) == "<UserPermission: perm>"

    def test_get(self, db_session):
        user = add_user(db_session)
        perm = UserPermissionService.get(
            user_id=user.id, perm_name="root", db_session=db_session
        )
        assert perm.user_id == user.id
        assert perm.perm_name == "root"

    def test_by_user_and_perm(self, db_session):
        add_user(db_session)
        user_permission = UserPermissionService.by_user_and_perm(
            1, "root", db_session=db_session
        )

        assert user_permission.user_id == 1
        assert user_permission.perm_name == "root"

    def test_by_user_and_perm_wrong_username(self, db_session):
        add_user(db_session)
        user_permission = UserPermissionService.by_user_and_perm(
            999, "root", db_session=db_session
        )

        assert user_permission is None

    def test_by_user_and_perm_wrong_permname(self, db_session):
        add_user(db_session)
        user_permission = UserPermissionService.by_user_and_perm(
            1, "wrong", db_session=db_session
        )

        assert user_permission is None



# -*- coding: utf-8 -*-
from __future__ import with_statement, unicode_literals

import os

import pytest

from ziggurat_foundations.models.services import BaseService
from ziggurat_foundations.tests import BaseTestCase
from ziggurat_foundations.tests.conftest import User

not_multi_db = os.environ.get("DB_STRING", "") == os.environ.get("DB_STRING2", "")


def add_user(i, db_name, db_session):
    user = User(
        id=i,
        user_name="username_{}_db_{}".format(i, db_name),
        email="email_{}".format(i),
        status=0,
    )
    db_session.add(user)


@pytest.mark.skipif(not_multi_db, reason="requires different db strings")
class TestMultiDB(BaseTestCase):
    def test_users_in_two_databases(self, db_session, db_session2):
        for x in range(1, 5):
            add_user(x, "a", db_session=db_session)
        for x in range(1, 3):
            add_user(x, "b", db_session=db_session2)
        db_session.flush()
        db_session2.flush()

        db_users = BaseService.all(User, db_session=db_session).all()
        db2_users = BaseService.all(User, db_session=db_session2).all()
        assert len(db_users) == 4
        assert len(db2_users) == 2
        assert db_users[0].user_name == "username_1_db_a"
        assert db2_users[0].user_name == "username_1_db_b"



"""create indices on resource owners

Revision ID: 613e7c11dead
Revises: b5e6dd3449dd
Create Date: 2018-02-15 11:51:29.659352

"""
from __future__ import unicode_literals

from alembic import op

# revision identifiers, used by Alembic.
revision = "613e7c11dead"
down_revision = "b5e6dd3449dd"


def upgrade():
    op.create_index(
        op.f("ix_resources_owner_group_id"), "resources", ["owner_group_id"]
    )
    op.create_index(op.f("ix_resources_owner_user_id"), "resources", ["owner_user_id"])


def downgrade():
    pass



# -*- coding: utf-8 -*-
from __future__ import with_statement, unicode_literals

import pprint

import pytest

from ziggurat_foundations.models.services.resource import ResourceService
from ziggurat_foundations.models.services.resource_tree import ResourceTreeService
from ziggurat_foundations.models.services.resource_tree_postgres import (
    ResourceTreeServicePostgreSQL,
)
from ziggurat_foundations.tests import add_resource, BaseTestCase, create_default_tree
from ziggurat_foundations.tests.conftest import not_postgres

tree_service = ResourceTreeService(ResourceTreeServicePostgreSQL)


class TestResources(BaseTestCase):
    def test_get(self, db_session):
        add_resource(db_session, 1, "root")
        resource = ResourceService.get(resource_id=1, db_session=db_session)
        assert resource.resource_id == 1
        assert resource.resource_name == "root"

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_root_nesting(self, db_session):
        root = create_default_tree(db_session)[0]

        result = tree_service.from_resource_deeper(
            root.resource_id, db_session=db_session
        )
        tree = tree_service.build_subtree_strut(result)
        tree_struct = tree["children"][-1]
        pprint.pprint(tree_struct)
        assert tree_struct["node"].resource_id == -1
        l1_nodes = [n for n in tree_struct["children"].values()]
        a_node = tree_struct["children"][1]
        b_node = tree_struct["children"][2]
        ac_node = a_node["children"][7]
        l_a_nodes = [n for n in a_node["children"].values()]
        l_b_nodes = [n for n in b_node["children"].values()]
        l_ac_nodes = [n for n in ac_node["children"].values()]
        assert [n["node"].resource_id for n in l1_nodes] == [1, 2, 3, 10, 11]
        assert [n["node"].resource_id for n in l_a_nodes] == [5, 6, 7, 8]
        assert [n["node"].resource_id for n in l_b_nodes] == [4]
        assert [n["node"].resource_id for n in l_ac_nodes] == [9]

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_full_nesting(self, db_session):
        create_default_tree(db_session)

        result = tree_service.from_parent_deeper(db_session=db_session)
        tree_struct = tree_service.build_subtree_strut(result)
        pprint.pprint(tree_struct)
        root_nodes = [n for n in tree_struct["children"].values()]
        l1_nodes = [n for n in tree_struct["children"][-1]["children"].values()]
        assert [n["node"].resource_id for n in root_nodes] == [-1, -2, -3]
        assert [n["node"].resource_id for n in l1_nodes] == [1, 2, 3, 10, 11]

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_branch_data_with_limit(self, db_session):
        create_default_tree(db_session)
        result = tree_service.from_resource_deeper(
            1, limit_depth=2, db_session=db_session
        )
        tree_struct = tree_service.build_subtree_strut(result)["children"][1]
        pprint.pprint(tree_struct)
        assert tree_struct["node"].resource_id == 1
        l_a_nodes = [n for n in tree_struct["children"].values()]
        assert [n["node"].resource_id for n in l_a_nodes] == [5, 6, 7, 8]
        assert len(tree_struct["children"][7]["children"].values()) == 0

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_branch_data_with_limit_from_parent(self, db_session):
        create_default_tree(db_session)
        result = tree_service.from_parent_deeper(
            1, limit_depth=2, db_session=db_session
        )
        tree_struct = tree_service.build_subtree_strut(result)
        pprint.pprint(tree_struct)
        l_a_nodes = [n for n in tree_struct["children"].values()]
        assert [n["node"].resource_id for n in l_a_nodes] == [5, 6, 7, 8]
        elem = tree_struct["children"][7]["children"]
        assert len(elem[9]["children"].values()) == 0

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_going_up_hierarchy(self, db_session):
        create_default_tree(db_session=db_session)
        result = tree_service.path_upper(9, db_session=db_session)
        assert [r.resource_id for r in result] == [9, 7, 1, -1]

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_up_on_same_branch(self, db_session):
        root = create_default_tree(db_session=db_session)[0]
        tree_service.move_to_position(3, to_position=2, db_session=db_session)
        result = tree_service.from_resource_deeper(
            root.resource_id, limit_depth=2, db_session=db_session
        )
        tree = tree_service.build_subtree_strut(result)
        tree = tree["children"][root.resource_id]
        pprint.pprint(tree)
        assert tree["children"][3]["node"].ordering == 2
        assert tree["children"][2]["node"].ordering == 3

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_up_on_same_branch_first(self, db_session):
        root = create_default_tree(db_session=db_session)[0]
        tree_service.move_to_position(3, to_position=1, db_session=db_session)
        result = tree_service.from_resource_deeper(
            root.resource_id, limit_depth=2, db_session=db_session
        )
        tree = tree_service.build_subtree_strut(result)
        tree = tree["children"][root.resource_id]
        pprint.pprint(tree)
        assert tree["children"][3]["node"].ordering == 1
        assert tree["children"][1]["node"].ordering == 2
        assert tree["children"][2]["node"].ordering == 3
        assert tree["children"][10]["node"].ordering == 4
        assert tree["children"][11]["node"].ordering == 5

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_down_on_same_branch(self, db_session):
        root = create_default_tree(db_session=db_session)[0]
        tree_service.move_to_position(1, to_position=3, db_session=db_session)
        result = tree_service.from_resource_deeper(
            root.resource_id, limit_depth=2, db_session=db_session
        )
        tree = tree_service.build_subtree_strut(result)
        tree = tree["children"][root.resource_id]
        pprint.pprint(tree)
        assert tree["children"][2]["node"].ordering == 1
        assert tree["children"][3]["node"].ordering == 2
        assert tree["children"][1]["node"].ordering == 3

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_down_on_same_branch_witn_new_parent_set(self, db_session):
        root = create_default_tree(db_session=db_session)[0]
        tree_service.move_to_position(
            1, to_position=3, new_parent_id=-1, db_session=db_session
        )
        result = tree_service.from_resource_deeper(
            root.resource_id, limit_depth=2, db_session=db_session
        )
        tree = tree_service.build_subtree_strut(result)
        tree = tree["children"][root.resource_id]
        pprint.pprint(tree)
        assert tree["children"][2]["node"].ordering == 1
        assert tree["children"][3]["node"].ordering == 2
        assert tree["children"][1]["node"].ordering == 3

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_down_on_same_branch_last(self, db_session):
        root = create_default_tree(db_session=db_session)[0]
        tree_service.move_to_position(1, to_position=5, db_session=db_session)
        result = tree_service.from_resource_deeper(
            root.resource_id, limit_depth=2, db_session=db_session
        )
        tree = tree_service.build_subtree_strut(result)
        tree = tree["children"][root.resource_id]
        pprint.pprint(tree)
        assert tree["children"][2]["node"].ordering == 1
        assert tree["children"][3]["node"].ordering == 2
        assert tree["children"][10]["node"].ordering == 3
        assert tree["children"][11]["node"].ordering == 4
        assert tree["children"][1]["node"].ordering == 5

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_down_on_same_branch_last_on_root(self, db_session):
        create_default_tree(db_session=db_session)[0]
        tree_service.move_to_position(-2, to_position=3, db_session=db_session)
        result = tree_service.from_parent_deeper(
            None, limit_depth=2, db_session=db_session
        )
        tree = tree_service.build_subtree_strut(result)
        pprint.pprint(tree)
        assert tree["children"][-1]["node"].ordering == 1
        assert tree["children"][-3]["node"].ordering == 2
        assert tree["children"][-2]["node"].ordering == 3

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_before_1_on_same_branch(self, db_session):
        from ziggurat_foundations.exc import ZigguratResourceOutOfBoundaryException

        create_default_tree(db_session=db_session)
        with pytest.raises(ZigguratResourceOutOfBoundaryException):
            tree_service.move_to_position(3, to_position=0, db_session=db_session)

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_after_last_on_same_branch(self, db_session):
        from ziggurat_foundations.exc import ZigguratResourceOutOfBoundaryException

        root = create_default_tree(db_session=db_session)[0]
        result = tree_service.from_resource_deeper(
            root.resource_id, limit_depth=2, db_session=db_session
        )
        tree = tree_service.build_subtree_strut(result)
        tree = tree["children"][root.resource_id]
        pprint.pprint(tree)
        with pytest.raises(ZigguratResourceOutOfBoundaryException):
            tree_service.move_to_position(3, to_position=6, db_session=db_session)

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_to_same_position(self, db_session):
        root = create_default_tree(db_session=db_session)[0]
        tree_service.move_to_position(1, to_position=1, db_session=db_session)
        result = tree_service.from_resource_deeper(
            root.resource_id, limit_depth=2, db_session=db_session
        )
        tree = tree_service.build_subtree_strut(result)["children"][root.resource_id]
        assert tree["children"][1]["node"].ordering == 1
        assert tree["children"][2]["node"].ordering == 2
        assert tree["children"][3]["node"].ordering == 3

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_inside_itself(self, db_session):
        from ziggurat_foundations.exc import ZigguratResourceTreePathException

        create_default_tree(db_session=db_session)
        with pytest.raises(ZigguratResourceTreePathException):
            tree_service.move_to_position(
                1, to_position=1, new_parent_id=6, db_session=db_session
            )

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_wrong_new_parent(self, db_session):
        from ziggurat_foundations.exc import ZigguratResourceTreeMissingException

        create_default_tree(db_session=db_session)
        with pytest.raises(ZigguratResourceTreeMissingException):
            tree_service.move_to_position(
                1, to_position=1, new_parent_id=-6, db_session=db_session
            )

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_to_same_parent(self, db_session):
        root = create_default_tree(db_session=db_session)[0]
        tree_service.move_to_position(
            1, 1, new_parent_id=root.resource_id, db_session=db_session
        )
        result = tree_service.from_resource_deeper(
            root.resource_id, limit_depth=2, db_session=db_session
        )
        tree = tree_service.build_subtree_strut(result)["children"][root.resource_id]
        assert tree["children"][1]["node"].ordering == 1
        assert tree["children"][2]["node"].ordering == 2
        assert tree["children"][3]["node"].ordering == 3

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_on_different_branch_with_siblings(self, db_session):
        create_default_tree(db_session)
        tree_service.move_to_position(
            6, new_parent_id=-1, to_position=1, db_session=db_session
        )

        result = tree_service.from_resource_deeper(
            -1, limit_depth=3, db_session=db_session
        )
        tree = tree_service.build_subtree_strut(result)
        tree_struct = tree["children"][-1]
        pprint.pprint(tree_struct)
        l_r_nodes = [n for n in tree_struct["children"].values()]
        check_values = [6, 1, 2, 3, 10, 11]
        assert [n["node"].resource_id for n in l_r_nodes] == check_values
        l_a_nodes = [n for n in tree_struct["children"][1]["children"].values()]
        pprint.pprint(l_a_nodes)
        assert [n["node"].resource_id for n in l_a_nodes] == [5, 7, 8]
        assert [n["node"].ordering for n in l_a_nodes] == [1, 2, 3]

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_after_last_on_different_branch(self, db_session):
        from ziggurat_foundations.exc import ZigguratResourceOutOfBoundaryException

        create_default_tree(db_session=db_session)
        with pytest.raises(ZigguratResourceOutOfBoundaryException):
            tree_service.move_to_position(
                4, new_parent_id=-1, to_position=7, db_session=db_session
            )

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_to_first_on_different_branch(self, db_session):
        create_default_tree(db_session)
        tree_service.move_to_position(
            4, new_parent_id=1, to_position=1, db_session=db_session
        )

        result = tree_service.from_resource_deeper(
            1, limit_depth=2, db_session=db_session
        )
        tree_struct = tree_service.build_subtree_strut(result)["children"][1]
        pprint.pprint(tree_struct)
        l_a_nodes = [n for n in tree_struct["children"].values()]
        assert [n["node"].resource_id for n in l_a_nodes] == [4, 5, 6, 7, 8]

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_to_middle_on_different_branch(self, db_session):
        create_default_tree(db_session)
        tree_service.move_to_position(
            4, new_parent_id=1, to_position=3, db_session=db_session
        )

        result = tree_service.from_resource_deeper(
            1, limit_depth=2, db_session=db_session
        )
        tree_struct = tree_service.build_subtree_strut(result)["children"][1]
        pprint.pprint(tree_struct)
        l_a_nodes = [n for n in tree_struct["children"].values()]
        assert [n["node"].resource_id for n in l_a_nodes] == [5, 6, 4, 7, 8]

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_to_last_on_different_branch(self, db_session):
        create_default_tree(db_session)
        tree_service.move_to_position(
            4, new_parent_id=1, to_position=5, db_session=db_session
        )

        result = tree_service.from_resource_deeper(
            1, limit_depth=2, db_session=db_session
        )
        tree_struct = tree_service.build_subtree_strut(result)["children"][1]
        pprint.pprint(tree_struct)
        l_a_nodes = [n for n in tree_struct["children"].values()]
        assert [n["node"].resource_id for n in l_a_nodes] == [5, 6, 7, 8, 4]

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_to_first_on_root_branch(self, db_session):
        create_default_tree(db_session)
        tree_service.move_to_position(
            4, new_parent_id=None, to_position=1, db_session=db_session
        )

        result = tree_service.from_parent_deeper(parent_id=None, db_session=db_session)
        tree_struct = tree_service.build_subtree_strut(result)["children"]
        pprint.pprint(tree_struct)
        r_nodes = [n for n in tree_struct.values()]
        assert [n["node"].resource_id for n in r_nodes] == [4, -1, -2, -3]

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    def test_move_from_root_deeper(self, db_session):
        create_default_tree(db_session)
        tree_service.move_to_position(
            -2, new_parent_id=1, to_position=1, db_session=db_session
        )

        result = tree_service.from_parent_deeper(parent_id=None, db_session=db_session)
        tree_struct = tree_service.build_subtree_strut(result)["children"]
        pprint.pprint(tree_struct)
        r_nodes = [n for n in tree_struct[-1]["children"][1]["children"].values()]
        assert [n["node"].resource_id for n in r_nodes] == [-2, 5, 6, 7, 8]
        assert list(tree_struct.keys()) == [-1, -3]
        assert [n["node"].ordering for n in tree_struct.values()] == [1, 2]

    @pytest.mark.skipif(not_postgres, reason="requires postgres")
    @pytest.mark.parametrize(
        "resource_id, expected_ids, expected_order",
        [
            (1, [-1, 2, 4, 3, 10, 11, -2, -3], None),
            (9, [-1, 1, 5, 6, 7, 8, 2, 4, 3, 10, 11, -2, -3], None),
            (12, [-1, 1, 5, 6, 7, 9, 8, 2, 4, 3, 10, 11, -2, -3], None),
            (-1, [-2, -3], [1, 2]),
        ],
    )
    def test_delete_branches(
        self, db_session, resource_id, expected_ids, expected_order
    ):
        create_default_tree(db_session)
        tree_service.delete_branch(resource_id, db_session=db_session)

        result = tree_service.from_parent_deeper(parent_id=None, db_session=db_session)
        row_ids = [r.Resource.resource_id for r in result]
        ordering = [r.Resource.ordering for r in result]
        assert row_ids == expected_ids
        if expected_order:
            assert ordering == expected_order



# -*- coding: utf-8 -*-
"""
Utility functions.
"""
import random
import string


class PlaceholderPasswordChecker(object):
    """Match and return false on check() for '*' (a password hash
    consisting of a single asterisk.) DelegatingPasswordManager would
    otherwise throw an exception 'unrecognized password hash'.
    """

    def match(self, encoded):
        return encoded == "*"

    def check(self, encoded, password):
        return False


class ModelProxy(dict):
    """
    Holds model references used in services
    """

    def __setattr__(self, key, value):
        self[key] = value

    def __getattr__(self, key):
        return self[key]


class NOOP(object):
    """
    For tree manager
    """

    def __nonzero__(self):
        return False

        # py3 compat

    __bool__ = __nonzero__


noop = NOOP()


def generate_random_string(chars=7):
    """

    :param chars:
    :return:
    """
    return u"".join(random.sample(string.ascii_letters * 2 + string.digits, chars))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import pkg_resources

from ziggurat_foundations.utils import ModelProxy, noop  # noqa

__version__ = pkg_resources.get_distribution("ziggurat_foundations").parsed_version


def import_model_service_mappings():
    from ziggurat_foundations.models.services.user import UserService
    from ziggurat_foundations.models.services.group import GroupService
    from ziggurat_foundations.models.services.group_permission import (
        GroupPermissionService,
    )
    from ziggurat_foundations.models.services.user_permission import (
        UserPermissionService,
    )
    from ziggurat_foundations.models.services.user_resource_permission import (
        UserResourcePermissionService,
    )
    from ziggurat_foundations.models.services.group_resource_permission import (
        GroupResourcePermissionService,
    )  # noqa
    from ziggurat_foundations.models.services.resource import ResourceService
    from ziggurat_foundations.models.services.resource_tree import ResourceTreeService
    from ziggurat_foundations.models.services.external_identity import (
        ExternalIdentityService,
    )

    return {
        "User": [UserService],
        "Group": [GroupService],
        "GroupPermission": [GroupPermissionService],
        "UserPermission": [UserPermissionService],
        "UserResourcePermission": [UserResourcePermissionService],
        "GroupResourcePermission": [GroupResourcePermissionService],
        "Resource": [ResourceService, ResourceTreeService],
        "ExternalIdentity": [ExternalIdentityService],
    }


def make_passwordmanager(schemes=None):
    """
    schemes contains a list of replace this list with the hash(es) you wish
    to support.
    this example sets pbkdf2_sha256 as the default,
    with support for legacy bcrypt hashes.

    :param schemes:
    :return: CryptContext()
    """
    from passlib.context import CryptContext

    if not schemes:
        schemes = ["pbkdf2_sha256", "bcrypt"]
    pwd_context = CryptContext(schemes=schemes, deprecated="auto")
    return pwd_context


def ziggurat_model_init(
    user=None,
    group=None,
    user_group=None,
    group_permission=None,
    user_permission=None,
    user_resource_permission=None,
    group_resource_permission=None,
    resource=None,
    external_identity=None,
    *args,
    **kwargs
):
    """
    This function handles attaching model to service if model has one specified
    as `_ziggurat_service`, Also attached a proxy object holding all model
    definitions that services might use

    :param args:
    :param kwargs:
    :param passwordmanager, the password manager to override default one
    :param passwordmanager_schemes, list of schemes for default
            passwordmanager to use
    :return:
    """
    models = ModelProxy()
    models.User = user
    models.Group = group
    models.UserGroup = user_group
    models.GroupPermission = group_permission
    models.UserPermission = user_permission
    models.UserResourcePermission = user_resource_permission
    models.GroupResourcePermission = group_resource_permission
    models.Resource = resource
    models.ExternalIdentity = external_identity

    model_service_mapping = import_model_service_mappings()

    if kwargs.get("passwordmanager"):
        user.passwordmanager = kwargs["passwordmanager"]
    else:
        user.passwordmanager = make_passwordmanager(
            kwargs.get("passwordmanager_schemes")
        )

    for name, cls in models.items():
        # if model has a manager attached attached the class also to manager
        services = model_service_mapping.get(name, [])
        for service in services:
            setattr(service, "model", cls)
            setattr(service, "models_proxy", models)



"""make password hash field bigger

Revision ID: 46a9c4fb9560
Revises: 5c84d7260c5
Create Date: 2011-12-20 17:59:16.961112

"""
from __future__ import unicode_literals

import sqlalchemy as sa
from alembic import op

# downgrade revision identifier, used by Alembic.
revision = "46a9c4fb9560"
down_revision = "5c84d7260c5"


def upgrade():
    op.alter_column(
        "users", "user_password", type_=sa.Unicode(256), existing_type=sa.Unicode(40)
    )
    op.alter_column(
        "users", "security_code", type_=sa.Unicode(256), existing_type=sa.Unicode(40)
    )


def downgrade():
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from ziggurat_foundations.utils import noop

__all__ = ["ResourceTreeService"]


class ResourceTreeService(object):
    model = None

    def __init__(self, service_cls):
        service_cls.model = self.model
        self.service = service_cls

    def from_resource_deeper(
        self, resource_id=None, limit_depth=1000000, db_session=None, *args, **kwargs
    ):
        """
        This returns you subtree of ordered objects relative
        to the start resource_id (currently only implemented in postgresql)

        :param resource_id:
        :param limit_depth:
        :param db_session:
        :return:
        """
        return self.service.from_resource_deeper(
            resource_id=resource_id,
            limit_depth=limit_depth,
            db_session=db_session,
            *args,
            **kwargs
        )

    def delete_branch(self, resource_id=None, db_session=None, *args, **kwargs):
        """
        This deletes whole branch with children starting from resource_id

        :param resource_id:
        :param db_session:
        :return:
        """
        return self.service.delete_branch(
            resource_id=resource_id, db_session=db_session, *args, **kwargs
        )

    def from_parent_deeper(
        self, parent_id=None, limit_depth=1000000, db_session=None, *args, **kwargs
    ):
        """
        This returns you subtree of ordered objects relative
        to the start parent_id (currently only implemented in postgresql)

        :param resource_id:
        :param limit_depth:
        :param db_session:
        :return:
        """
        return self.service.from_parent_deeper(
            parent_id=parent_id,
            limit_depth=limit_depth,
            db_session=db_session,
            *args,
            **kwargs
        )

    def build_subtree_strut(self, result, *args, **kwargs):
        """
        Returns a dictionary in form of
        {node:Resource, children:{node_id: Resource}}

        :param result:
        :return:
        """
        return self.service.build_subtree_strut(result=result, *args, **kwargs)

    def path_upper(
        self, object_id, limit_depth=1000000, db_session=None, *args, **kwargs
    ):
        """
        This returns you path to root node starting from object_id
            currently only for postgresql

        :param object_id:
        :param limit_depth:
        :param db_session:
        :return:
        """
        return self.service.path_upper(
            object_id=object_id,
            limit_depth=limit_depth,
            db_session=db_session,
            *args,
            **kwargs
        )

    def move_to_position(
        self,
        resource_id,
        to_position,
        new_parent_id=noop,
        db_session=None,
        *args,
        **kwargs
    ):
        """
        Moves node to new location in the tree

        :param resource_id: resource to move
        :param to_position: new position
        :param new_parent_id: new parent id
        :param db_session:
        :return:
        """
        return self.service.move_to_position(
            resource_id=resource_id,
            to_position=to_position,
            new_parent_id=new_parent_id,
            db_session=db_session,
            *args,
            **kwargs
        )

    def shift_ordering_down(
        self, parent_id, position, db_session=None, *args, **kwargs
    ):
        """
        Shifts ordering to "close gaps" after node deletion or being moved
        to another branch, begins the shift from given position

        :param parent_id:
        :param position:
        :param db_session:
        :return:
        """
        return self.service.shift_ordering_down(
            parent_id=parent_id,
            position=position,
            db_session=db_session,
            *args,
            **kwargs
        )

    def shift_ordering_up(self, parent_id, position, db_session=None, *args, **kwargs):
        """
        Shifts ordering to "open a gap" for node insertion,
        begins the shift from given position

        :param parent_id:
        :param position:
        :param db_session:
        :return:
        """
        return self.service.shift_ordering_up(
            parent_id=parent_id,
            position=position,
            db_session=db_session,
            *args,
            **kwargs
        )

    def set_position(self, resource_id, to_position, db_session=None, *args, **kwargs):
        """
        Sets node position for new node in the tree

        :param resource_id: resource to move
        :param to_position: new position
        :param db_session:
        :return:def count_children(cls, resource_id, db_session=None):
        """
        return self.service.set_position(
            resource_id=resource_id,
            to_position=to_position,
            db_session=db_session,
            *args,
            **kwargs
        )

    def check_node_parent(
        self, resource_id, new_parent_id, db_session=None, *args, **kwargs
    ):
        """
        Checks if parent destination is valid for node

        :param resource_id:
        :param new_parent_id:
        :param db_session:
        :return:
        """
        return self.service.check_node_parent(
            resource_id=resource_id,
            new_parent_id=new_parent_id,
            db_session=db_session,
            *args,
            **kwargs
        )

    def count_children(self, resource_id, db_session=None, *args, **kwargs):
        """
        Counts children of resource node

        :param resource_id:
        :param db_session:
        :return:
        """
        return self.service.count_children(
            resource_id=resource_id, db_session=db_session, *args, **kwargs
        )

    def check_node_position(
        self, parent_id, position, on_same_branch, db_session=None, *args, **kwargs
    ):
        """
        Checks if node position for given parent is valid, raises exception if
        this is not the case

        :param parent_id:
        :param position:
        :param on_same_branch: indicates that we are checking same branch
        :param db_session:
        :return:
        """
        return self.service.check_node_position(
            parent_id=parent_id,
            position=position,
            on_same_branch=on_same_branch,
            db_session=db_session,
            *args,
            **kwargs
        )



"""add external identity tables

Revision ID: 24ab8d11f014
Revises: 2bb1ba973f0b
Create Date: 2011-11-10 23:18:19.446844

"""
from __future__ import unicode_literals

import sqlalchemy as sa
from alembic import op

# downgrade revision identifier, used by Alembic.
revision = "24ab8d11f014"
down_revision = "2bb1ba973f0b"


def upgrade():
    op.create_table(
        "external_identities",
        sa.Column("external_id", sa.Unicode(255), primary_key=True),
        sa.Column("external_user_name", sa.Unicode(50), default=""),
        sa.Column(
            "local_user_name",
            sa.Unicode(50),
            sa.ForeignKey("users.user_name", onupdate="CASCADE", ondelete="CASCADE"),
            primary_key=True,
        ),
        sa.Column("provider_name", sa.Unicode(50), default="", primary_key=True),
        sa.Column("access_token", sa.Unicode(255), default=""),
        sa.Column("alt_token", sa.Unicode(255), default=""),
        sa.Column("token_secret", sa.Unicode(255), default=""),
    )


def downgrade():
    op.drop_table("external_identities")
